/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: SignStatus.java 10718 2006-11-29 02:54:54Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applications.signerclient;

import java.awt.Toolkit;
import java.io.BufferedInputStream;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JLabel;

/**
 * @version $Rev: 10718 $ $Date: 2006-11-29 09:54:54 +0700 (Wed, 29 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
class SignStatus {

	private static final String SIGNING_IMAGE = "/images/signing.jpg";
	private static final String SUCCESS_IMAGE = "/images/ok.jpg";
	private static final String FAILURE_IMAGE = "/images/fail.jpg";

	private JLabel showing;

    /**
     * @param logger 
     * @param width
     * @param height
     */
    public SignStatus(int width, int height) {
	    super();

	    showing = new JLabel("Loading images...");
	    showing.setBounds(0, 0, width, height);
	    showing.setHorizontalAlignment(JLabel.CENTER);
	    showing.setVerticalAlignment(JLabel.CENTER);
	    showing.setHorizontalTextPosition(JLabel.CENTER);
	    showing.setVerticalTextPosition(JLabel.CENTER);
    }

	private static final long serialVersionUID = 1L;

	public JLabel getShowingLabel() {
		return showing;
	}

	private void showImage(String image) {
		ImageIcon imgSrc = loadImage(image);

		showing.setIcon(imgSrc);
		showing.setText("");

		showing.repaint();
	}

	public void showSigning() {
		showImage(SIGNING_IMAGE);
	}

	public void showSuccess() {
		showImage(SUCCESS_IMAGE);
	}

	public void showFailure() {
		showImage(FAILURE_IMAGE);
	}

	private static ImageIcon loadImage(String image) {

		ImageIcon icon = null;

		BufferedInputStream istream = new BufferedInputStream(
				SignStatus.class.getResourceAsStream(image));

		if (istream != null) {
			byte[] buffer = new byte[16384];

			try {
				int count = istream.read(buffer);

				if (count == 0)
					return null;

            } catch (IOException e) {
	            return null;
            } finally {
            	try {
	                istream.close();
                } catch (IOException e) {
	                return null;
                }
            }

            icon = new ImageIcon(Toolkit.getDefaultToolkit().createImage(buffer));
            buffer = null;

		}

		return icon;
	}
}
