package org.exoplatform.faces.cms;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.Value;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.xml.PortalContainerInfo;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;

public class CMSDelegate implements CMSViewComponent {

  private String portalName_;
  private WikiContext wikiContext;
  private WikiEngineService service_;
  private Node node_;

  public CMSDelegate(Node node) {
    this.node_ = node;
    PortalContainer pcontainer = PortalContainer.getInstance();
    PortalContainerInfo info = 
      (PortalContainerInfo) pcontainer.getComponentInstanceOfType(PortalContainerInfo.class);
    
    portalName_ = info.getContainerName();

    service_ = (WikiEngineService) pcontainer.getComponentInstanceOfType(WikiEngineService.class);
  }

  public String getPortalName() {
    return portalName_;
  }

  public String getWorkspaceName() throws Exception {
    return node_.getSession().getWorkspace().getName();
  }

  public Node getNode() throws Exception {
    return node_;
  }

  public void setNode(Node node) {
    this.node_ = node;
  }

  public boolean isNodeTypeSupported() {
    try {      
      TemplateService templateService = 
        (TemplateService) PortalContainer.getComponent(TemplateService.class);
      String nodeTypeName = node_.getPrimaryNodeType().getName();
      return templateService.isManagedNodeType(nodeTypeName);
    } catch (Exception e) {
      return false;
    }
  }

  public String getNodeType() throws Exception {
    return node_.getPrimaryNodeType().getName();
  }
  
  public Object getComponentInstanceOfType(String className) {
    Object service = null;
    try {
      ClassLoader loader =  Thread.currentThread().getContextClassLoader();
      Class clazz = loader.loadClass(className);
      PortalContainer container = PortalContainer.getInstance();
      service = container.getComponentInstanceOfType(clazz);
    } catch (ClassNotFoundException ex) {
      ex.printStackTrace();
    } 
    return service;
  }

  public WikiContext getWikiContext() {
    if (wikiContext == null) {
      wikiContext = new WikiContext();
      wikiContext.put("portalHook", new PortalHook());
    }
    return wikiContext;
  }

  public WikiEngineService getWikiService() {
    return service_;
  }
  
  public boolean isRssLink() { return false ; }
  public String getRssLink() { return null ; }
  
  public String getTemplatePath() throws Exception {
    String nodeTypeName = node_.getPrimaryNodeType().getName();
    PortalContainer container = PortalContainer.getInstance();
    TemplateService templateService = 
      (TemplateService) container.getComponentInstanceOfType(TemplateService.class);
    return templateService.getTemplatePath(false, nodeTypeName);

  }
  
  public List<Node> getRelations() throws Exception {
  	List<Node> relations = new ArrayList<Node>();
  	try {
  	  Value[] vals = node_.getProperty("exo:relation").getValues();
  	  for (int i = 0; i < vals.length; i++) {
  		String uuid = vals[i].getString();
  		Node currentNode = node_.getSession().getNodeByUUID(uuid);
  		relations.add(currentNode);
  	  }
  	} catch (Exception e) {
  	}
  	return relations;
  }

  public List<Node> getAttachments() throws Exception {
	List<Node> attachments = new ArrayList<Node>();
	String nodeType = "";
	NodeIterator childrenIterator;
	childrenIterator = node_.getNodes();
	while (childrenIterator.hasNext()) {
	  Node childNode = childrenIterator.nextNode();
	  try {
		nodeType = childNode.getPrimaryNodeType().getName();
		if ("nt:file".equals(nodeType))
		  attachments.add(childNode);
	  } catch (Exception e) {
	  }
	}
	return attachments;
  }  
  
  public String getAttachmentSize(Node node){
  	Node jcrContentNode = null;
  	int size = 0;
  	String sSize = "0";
  	String nodeType = "";
    try {      
      nodeType = node.getPrimaryNodeType().getName();     
    } catch (Exception ex) {
      nodeType = "default";
    }
  	try {
  		if ("nt:file".equals(nodeType))
        jcrContentNode = node.getNode("jcr:content");
  		else if ("nt:resource".equals(nodeType))
        jcrContentNode = node;
  		if (jcrContentNode != null) {
  			Property data = null;
  			if (jcrContentNode.hasProperty("jcr:data"))
  				data = jcrContentNode.getProperty("jcr:data");
  			
  			InputStream is = data.getValue().getStream();
  			byte[] buf = new byte[1024];
  			int len;
        while ((len = is.read(buf)) > 0) {
            size += len;
        }
        is.close();
  		}
  	} catch (Exception e) {
    }
  	
  	if(size >0){
  		sSize = String.valueOf((((float)(size/100))/10));
  	}
  	sSize += " Kb";
  	return sSize;
  }

  public boolean hasPropertyContent(Node node, String property){
    try {
      String value = node.getProperty(property).getString() ;
      if(value.length() > 0) return true ;
    }catch (Exception e) {
      e.printStackTrace() ;      
    }
    return false ;
  }

  public List getSupportedLocalise() throws Exception {
    List<String> local = new ArrayList<String> () ;
    if(node_.hasNode("languages")){
      Node languages = node_.getNode("languages") ;
      NodeIterator iter = languages.getNodes() ;
      while(iter.hasNext()) {
        local.add(iter.nextNode().getName()) ;
      }
      local.add(node_.getProperty("exo:language").getString()) ;      
    } 
    return local ;
  }

  public String getLargeNodeTypeIcon(Node node) {    
    return CMSUtils.getLargeNodeTypeIcon(node) ;
  }
  
  public String getSmallNodeTypeIcon(Node node) {
    return CMSUtils.getSmallNodeTypeIcon(node);
  }
  
  public String getPropertyIconByName(String propertyName) {
    return CMSUtils.getPropertyIconByName(propertyName); 
  }

  public String getPropertyTypeIcon(Property property) {
    return CMSUtils.getPropertyTypeIcon(property);
  }
}
