/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.List;
import java.util.ResourceBundle;
import java.util.Stack;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.portlet.PortletPreferences;

import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.folksonomy.component.UITagCloud;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;

/**
 * @email: nguyenkequanghung@yahoo.com
 * @version: $Id: UIExplorer.java,v 1.2 2004/07/31 15:07:15 nqhung Exp $
 */
public class UIContentExplorer extends UIExoCommand {  
  final static public String CHANGE_NODE_ACTION = "changeNode".intern();  
  final static public String SEARCH_ACTION = "search".intern() ;

  public static Parameter[] CHANGE_NODE = { new Parameter(ACTION, CHANGE_NODE_ACTION) };  
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION,  BACK_ACTION) };
  public static Parameter[] SEARCH_PARAMS = {new Parameter(ACTION,SEARCH_ACTION)} ;

  protected String rootPath_ ;
  protected Node rootNode_ ;
  protected Node currentNode_;
  protected Node previousNode_;  
  protected String usecase_ ;  

  protected Stack<Node> nodesHistory = new Stack<Node>();	
  private Session session_;                  	
  private boolean isShowingToolbar = true;
  private boolean isShowingTagsCloud = true ;
  private UIToolbar uiToolbar_ = new UIToolbar("MainToolbar"); 
  private UIContentList uiContentList_ ;
  private UIDetailDocument uiDetailDocument_ ;
  private UITaggedDocuments uiTaggedDocuments_ ;
  private UITagCloud uiTagCloud_ ;
  private String workspace_ ;
  private String contentListTemplate_ ;
  private String documentcontentTemplate_ ;

  public UIContentExplorer() throws Exception {      
    setId("UIContentExplorer");    
    setRendererType("VelocityRenderer") ;
    PortletPreferences prefs = FacesUtil.getPortletPreferences() ;				    
    isShowingToolbar = Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TOOLBAR,"")) ;
    isShowingTagsCloud = Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TAGS_CLOUD,"")) ;
    usecase_ = prefs.getValue(CBUtils.USECASE,"") ;
    
    uiToolbar_.addRightButton(new Button("#{UIContentExplorer.button.content-search}", SEARCH_PARAMS));
    uiToolbar_.addLeftButton(new Button("#{UIContentExplorer.button.back}",  BACK_PARAMS));    
    if(usecase_.equals(CBUtils.FORM_PATH)) {
      if(isShowingToolbar){                
        uiToolbar_.setRendered(isShowingToolbar) ;
        getChildren().add(uiToolbar_);
      }
    }else {
      isShowingToolbar = false ;
    }         
           
    uiTagCloud_ = (UITagCloud)addChild(UITagCloud.class) ;
    uiTagCloud_.setRendered(isShowingTagsCloud) ;    
    uiTaggedDocuments_ = (UITaggedDocuments)addChild(UITaggedDocuments.class) ;
    uiTaggedDocuments_.setRendered(false) ;
    uiContentList_ = (UIContentList)addChild(UIContentList.class) ;
    uiContentList_.setRendered(true) ;
    uiContentList_.setTemplatePath(contentListTemplate_) ;    
    uiDetailDocument_ = (UIDetailDocument)addChild(UIDetailDocument.class) ;
    uiDetailDocument_.setRendered(false) ;        
    uiDetailDocument_.setTemplatePath(documentcontentTemplate_) ;
    addChild(UISearch.class).setRendered(false);
        
    addActionListener(BackActionListener.class, BACK_ACTION);
    addActionListener(SearchActionListener.class, SEARCH_ACTION);
  }	

  public boolean isShowingToolbar() { return isShowingToolbar ; }
  public void setShowingToolbar(boolean isViewing) { 
    this.isShowingToolbar = isViewing ; 
  }

  public boolean isShowingTagsCloud() { return isShowingTagsCloud ; }
  public void setShowingTagsCloud(boolean isViewing) {
    this.isShowingTagsCloud = isViewing ;
  }
  
  public boolean hasToolBar() {    
    if(getToolbar() != null) return true ;
    return false ;
  }

  public Session getSession() { return session_; }	

  public void initContenExplorerData(Session session,String rootPath, 
      String contentListTemplate, String documentTemplate) throws Exception{
    session_ = session ;
    rootPath_ = rootPath ;    
    rootNode_ = (Node)session.getItem(rootPath_) ;
    uiContentList_.setTemplatePath(contentListTemplate) ;
    uiDetailDocument_.setTemplatePath(documentTemplate) ;
    setSelectNode(rootNode_) ;
  }

  public UIToolbar getToolbar() { return uiToolbar_ ; }
  
  public void initScriptContenExplorerData(Session session, List<Node> contentList, String workspace,
      String contentListTemplate, String documentTemplate) throws Exception{
    session_ = session ;
    rootNode_ = (Node)session.getItem("/") ;
    uiContentList_.setTemplatePath(contentListTemplate) ;
    uiContentList_.updateScriptData(contentList, workspace) ;
    uiDetailDocument_.setTemplatePath(documentTemplate) ;    
    setSelectNode(rootNode_) ;
  }

  public String getWorkspace() { return workspace_ ; }

  public Node getRootNode() {	return rootNode_ ; }	
  public String getRootPath() { return rootPath_ ; }						                
  public void setRootPath( String path ) { rootPath_  = path ; }

  public Node getSelectNode() { return currentNode_; }	
  public void setSelectNode(Node node) throws Exception {    
    record(node);          
    currentNode_ = node;
    broadcastOnChange(node);      
  }

  public boolean isReadAuthorized(Node node) {
    ExtendedNode extendedNode = (ExtendedNode)node ;
    try {
      extendedNode.checkPermission(PermissionType.READ) ;
    }catch (Exception e) {
      return false ;
    }
    return true ;
  }

  public void setSelectNode(String uri) throws Exception {    
    Node selectedNode = (Node)session_.getItem(uri) ;
    ExtendedNode extNode = (ExtendedNode)selectedNode ;
    extNode.checkPermission(PermissionType.READ) ;     
    if (uri.equals(rootPath_)){            
      currentNode_ = rootNode_;      
      previousNode_ = null ;      
    }else{
      previousNode_ = currentNode_ ;
      currentNode_ = selectedNode ;            
    }         
    if (previousNode_ != null) record(previousNode_);
    broadcastOnChange(currentNode_);          
  }

  public Node getSelectNode(String uri) throws Exception {
    Node node = (Node)session_.getItem(uri) ;
    return node ;
  }     

  public void broadcastOnChange(Node node) throws Exception {    
    List list = findDescendantsOfType(ChangeNodeListener.class);
    for (int i = 0; i < list.size(); i++) {
      ChangeNodeListener listener = (ChangeNodeListener) list.get(i);
      listener.onChange(this, node);
    }        
  }

  public void record(Node node) { nodesHistory.push(node); }

  public Node rewind() { return nodesHistory.pop(); }

  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIContentExplorer uicomp = (UIContentExplorer) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node previousNode = uicomp.rewind();
        uicomp.currentNode_ = previousNode;        
        uicomp.broadcastOnChange(previousNode);                
      } catch (Exception e) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UIContentExplorer.msg.no-node-history")));
      }
    }        
  }

  static public class SearchActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {
      UIContentExplorer uicomp = (UIContentExplorer)event.getComponent() ;
      UISearch uiSearch = (UISearch)uicomp.getChildComponentOfType(UISearch.class) ;
      boolean isRendered = uiSearch.isRendered() ;
      uiSearch.setRendered(!isRendered) ;
      UIContentList uiContentList = (UIContentList)uicomp.getChildComponentOfType(UIContentList.class) ;
      
      uiContentList.setRendered(isRendered) ;
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
      boolean isViewingTagsCloud = Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TAGS_CLOUD,"")) ;
      if(isViewingTagsCloud) {
        UITagCloud uiTagcloud = (UITagCloud)uicomp.getChildComponentOfType(UITagCloud.class);        
        uiTagcloud.setRendered(true) ;
      }
    }    
  } 
}
