package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.faces.context.ExternalContext;
import javax.jcr.AccessDeniedException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;
import javax.portlet.ActionResponse;
import javax.portlet.PortletPreferences;
import javax.portlet.WindowState;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.Utils;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.queries.QueryService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ExtendedNode;

/**
 * Created by The eXo Platform SARL
 * Author : HoaPham
 *          phamvuxuanhoa@yahoo.com
 * Jan 20, 2006
 */

public class UIContentList extends UIExoCommand implements ChangeNodeListener {
  public final static String VIEW_CATEGORY_ACTION="viewCategory".intern() ;
  public final static String VIEW_DOCUMENT_ACTION = "viewDocument".intern() ;
  public final static String REFRESH_ACTION = "refreshList".intern() ;  
  public static final int NB_CATEGORY_PER_ROW = 3; 
  public static final String ASCENDING_ORDER = "Ascending".intern();

  static Parameter[] VIEW_CATEGORY_PARAMS = {new Parameter(ACTION,VIEW_CATEGORY_ACTION)} ;
  static Parameter[] VIEW_DOCUMENT_PARAMS = {new Parameter(ACTION,VIEW_DOCUMENT_ACTION)} ;    
  static Parameter[] CHANGE_NODE_PARAMS = { new Parameter(ACTION, UIContentExplorer.CHANGE_NODE_ACTION) };          
  static Parameter[] REFRESH_PARAMS = {new Parameter(ACTION,REFRESH_ACTION)} ;

  private String rootPath_ ;
  protected Node node_;
  protected UIContentExplorer explorer_ ;
  private RepositoryService repositoryService_ ;  
  private UIPageListIterator uiPageListIterator_ ;
  private String workspace_ ;
  private String systemWorkspace_ ;
  private String templatePath_ ; 
  private boolean showRefDocument_ ;
  private boolean showChildDocument_ ;    
  private String usecase_ ;    

  private String queryStatement_ ;
  private String queryLanguage_ ;
  private String queryStore_ ;
  private int nbDocumentPerPage_ ;  
  private List<Document> documentList_ = new ArrayList<Document>();

  public UIContentList(RepositoryService repositoryService, CmsConfigurationService cmsConfig) throws Exception{

    setId("UIContentList");
    setRendererType("VelocityRenderer");			

    repositoryService_ = repositoryService ;        
    PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
    systemWorkspace_ = cmsConfig.getWorkspace() ;
    workspace_ = prefs.getValue(CBUtils.WORKSPACE , "") ;        	
    queryLanguage_ = prefs.getValue(CBUtils.QUERY_LANGUAGE , "") ;
    queryStatement_ = prefs.getValue(CBUtils.QUERY_STATEMENT , "") ;
    queryStore_ = prefs.getValue(CBUtils.QUERY_STORE , "") ;
    usecase_ = prefs.getValue(CBUtils.USECASE,"") ;

    try {
      nbDocumentPerPage_ = Integer.parseInt(prefs.getValue(CBUtils.NB_PER_PAGE , ""));
    } catch (Exception e) {
      nbDocumentPerPage_ = 5;
    }    
    showRefDocument_ = Boolean.parseBoolean(prefs.getValue(CBUtils.REF_DOCUMENT,"")) ;
    showChildDocument_ = Boolean.parseBoolean(prefs.getValue(CBUtils.CHILD_DOCUMENT,"")) ;				        
    uiPageListIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageListIterator_) ;    

    addActionListener(ChangeNodeActionListener.class, UIContentExplorer.CHANGE_NODE_ACTION);
    addActionListener(ViewCategoryActionListener.class,VIEW_CATEGORY_ACTION) ;
    addActionListener(ViewDocumentActionListener.class,VIEW_DOCUMENT_ACTION) ;
    addActionListener(RefreshListActionListener.class,REFRESH_ACTION) ;

  }   

  public Parameter[] getChangeNodeParams() { return CHANGE_NODE_PARAMS;	} 	
  public Parameter[] getViewCategoryParams() { return VIEW_CATEGORY_PARAMS;	}
  public Parameter[] getRefreshParams() { return REFRESH_PARAMS; }

  public Parameter[] getViewDocumentParams() {    			
    return VIEW_DOCUMENT_PARAMS;
  }		

  public void setTemplatePath(String templatePath) {
    this.templatePath_ = templatePath ;
  }

  public String getTemplateToView() {
    return "jcr:" + templatePath_ ;
  }

  public Node getNode() throws Exception { return node_; }  
  public String getQueryLanguage(){ return queryLanguage_ ; }
  public String getRootPath() {return rootPath_ ;}

  public void initContentListData ( Node node){
    try{
      rootPath_ = node.getPath() ;
    }catch(RepositoryException e){      
    }    
    node_ = node ;    
  }
  public String[] getSelectedNodePath() throws Exception {            
    String path = node_.getPath() ;        
    if(path.length()>= rootPath_.length()) {
      path = path.substring(rootPath_.length()) ;
    }
    String[] temp =  StringUtils.split(path, CBUtils.SLASH) ;    
    return temp ;
  }   

  public boolean hasChildren() { return hasChildren(null); }

  public boolean hasChildren(Node parent) {    
    try {
      if (parent == null){        
        return node_.hasNodes(); 
      }                    
      return parent.hasNodes();
    } catch (RepositoryException e) {
      return false;
    }    
  }

  public List getChildCategories(Node parent) throws Exception {
    return getCategoriesList(parent.getNodes()) ;    
  }		

  public List<Node> getCategoriesList(NodeIterator iter) throws Exception {
    ArrayList<Node> list = new ArrayList<Node>() ;
    NodeType nodeType = null ;
    while(iter.hasNext()) {
      Node childNode = iter.nextNode();                   
      nodeType = childNode.getPrimaryNodeType();                 
      if(isCategories(nodeType))        
        list.add(childNode) ;
    }
    return list ;    
  }

  public Collection getSiblingCategories() throws Exception {        
    NodeIterator iter = null;        
    if(node_.getPath().equals(rootPath_)) {
      iter = node_.getNodes();
    } else {
      iter = node_.getParent().getNodes();
    }    
    List nodes= getCategoriesList(iter) ;    
    return Utils.getChildrenPerRow(NB_CATEGORY_PER_ROW,nodes.iterator()) ;
  }

  public Collection <Node> getDocumentPageList() throws Exception {
    return uiPageListIterator_.getPageListDataHandler().getDatas() ;
  }		  

  public String getDocumentOwner(Document document) throws Exception{
    Node node = document.getNode() ;
    String owner = ((ExtendedNode) node).getACL().getOwner();    
    if(owner != null) return owner ;
    return "System";
  }

  public void onChange(UIContentExplorer uiExplorer, Node node) throws Exception {     
    node_ = node;
    explorer_ = uiExplorer ;
    rootPath_ = explorer_.getRootPath() ;  
    updateDocumentList() ;
    setRendered(true);
  }

  public void updateDocumentList() throws Exception {
    if(hasDocuments()) {
      List<Document> pageList = getDocuments() ;
      ObjectPageList objectPageList = new ObjectPageList(pageList,nbDocumentPerPage_) ;
      uiPageListIterator_.setPageList(objectPageList) ;
    }    
  }

  public void updateScriptData(List<Node> contentList, String workspace) throws Exception{
    List<Document> pageList = new ArrayList<Document>(contentList.size()) ;
    for(Node node : contentList) {
      Document doc = new Document(workspace,node) ;           
      pageList.add(doc) ;
    }
    documentList_ = pageList ;
    ObjectPageList objectPageList = new ObjectPageList(pageList,10) ;
    uiPageListIterator_.setPageList(objectPageList) ;    
  }

  public boolean isRootCategory() throws Exception{    
    if((node_.getPath().equals(rootPath_))||node_ == null) return true ;
    return false ;
  }        

  public List<Document>getAllDocuments() {
    Collections.sort(documentList_,new DocumentComparator(ASCENDING_ORDER)) ;
    return documentList_ ; 
  }

  public boolean hasDocuments() throws Exception {    
    if (usecase_.equals(CBUtils.FORM_PATH) || usecase_.equals(CBUtils.USE_SCRIPT)){            
      if(showRefDocument_) {
        if(isReferenceableNode(node_)) {
          String uuid = node_.getUUID() ;
          String[] workspaces = repositoryService_.getRepository().getWorkspaceNames() ;
          for(String workspace:workspaces) {
            Session session = repositoryService_.getRepository().getSystemSession(workspace) ;
            Node taxonomyNode = session.getNodeByUUID(uuid) ;
            PropertyIterator categoriesIter = taxonomyNode.getReferences() ;
            if(categoriesIter.getSize() > 0) return true ;
          }              
        }                  
      }    
      if(showChildDocument_){
        NodeIterator iter = node_.getNodes() ;
        while(iter.hasNext()) {        
          Node child = iter.nextNode() ;                    
          NodeType nt = child.getPrimaryNodeType() ;     
          if(CBUtils.isDocument(nt)) {            
            return true ;
          }                                 
        }
      }         
      return false ;
    }     
    try {
      if(getDocumentPageList().size() > 0 ) {
        return true;                                
      }   
      return false;
    }catch (Exception e) {      
      return true ;
    }   
  }  
  
  public String getNodeIcon(Node node)  throws Exception {    
    return CMSUtils.getSmallNodeTypeIcon(node) ;
  }
  
  public String getDocumentIcon(Document document) {    
    return  CMSUtils.getSmallNodeTypeIcon(document.getNode()) ;
  }
  
  public List<Document> getDocuments() throws Exception {                
    List<Document> refDocuments = new ArrayList<Document>() ;
    List<Document> childDocuments = new ArrayList<Document>() ;       
    documentList_.clear() ;
    if (usecase_.equals(CBUtils.FORM_PATH) || usecase_.equals(CBUtils.USE_SCRIPT)){      
      if(showRefDocument_) {      
        if(isReferenceableNode(node_)) {
          String uuid = node_.getUUID() ;
          String[] workspaces = repositoryService_.getRepository().getWorkspaceNames() ;
          for(String workspace:workspaces) {
            Session session = repositoryService_.getRepository().getSystemSession(workspace) ;
            Node taxonomyNode = session.getNodeByUUID(uuid) ;
            for(PropertyIterator categoriesIter = taxonomyNode.getReferences(); categoriesIter.hasNext();) {
              Property exoCategoryProp = categoriesIter.nextProperty();
              Node refNode = exoCategoryProp.getParent() ;
              Document refDoc = new Document(workspace,refNode) ;           
              refDocuments.add(refDoc) ;
            }
          }     
        }                                                                              
      }            
      if(showChildDocument_) {
        NodeIterator iter = node_.getNodes() ;       
        String workspace= node_.getSession().getWorkspace().getName() ;             
        while(iter.hasNext()) {      
          Node child = iter.nextNode() ;
          NodeType nt = child.getPrimaryNodeType() ;
          if(CBUtils.isDocument(nt)) {                   
            Document childDoc = new Document(workspace,child) ;
            childDocuments.add(childDoc) ;           
          }                                             
        }       
      }
      if(showRefDocument_ && showChildDocument_) {                     
        documentList_.addAll(refDocuments) ;
        documentList_.addAll(childDocuments) ;        
      }    
      else if(!showChildDocument_) {   
        documentList_ = refDocuments ;        
      }
      else {     
        documentList_ = childDocuments ;        
      }      
      Collections.sort(documentList_,new DocumentComparator(ASCENDING_ORDER)) ;
      return documentList_ ;
    } 
    if(usecase_.equals(CBUtils.USE_JCR_QUERY)) {
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
      boolean isNewQuery = Boolean.parseBoolean(prefs.getValue(CBUtils.IS_NEW_QUERY, "")) ;
      QueryResult queryResult = null;
      if(isNewQuery) {
        Session session = repositoryService_.getRepository().login(workspace_) ;
        QueryManager queryManager = session.getWorkspace().getQueryManager();
        Query query = queryManager.createQuery(queryStatement_, queryLanguage_);  
        queryResult = query.execute();
      }else {
        QueryService qservice = (QueryService)PortalContainer.getInstance().getComponentInstanceOfType(QueryService.class) ;
        if(queryStore_ != null && queryStore_.length() > 0) {
          queryResult = qservice.execute(queryStore_);
        }
      }
      NodeIterator iter = queryResult.getNodes();     
      while (iter.hasNext()) {              
        Node child = iter.nextNode() ;        
        NodeType nt = child.getPrimaryNodeType() ;
        if(CBUtils.isDocument(nt)) {
          Document childDoc = new Document(workspace_,child) ;
          documentList_.add(childDoc) ; 
        }
      }
    }
    Collections.sort(documentList_, new DocumentComparator(ASCENDING_ORDER)) ;
    return documentList_;    
  }

  private boolean isCategories(NodeType nodeType) {
    for(String type:CBUtils.CATEGORY_NODE_TYPES) {
      if(nodeType.isNodeType(type)) return true ;
    }
    return false ;
  }

  private boolean isReferenceableNode(Node node) throws Exception {
    NodeType[] nodeTypes = node.getMixinNodeTypes() ;
    for(NodeType type:nodeTypes) {      
      if(type.getName().equals(CBUtils.MIX_REFERENCEABLE)) return true ;
    }
    return false ;
  }

  public String getPortalName() {
    PortalContainer pcontainer = PortalContainer.getInstance();
    String portalName = pcontainer.getPortalContainerInfo().getContainerName() ;
    return portalName;
  }

  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIContentList uicomp = (UIContentList) event.getComponent();
      UIContentExplorer uiExplorer = 
        (UIContentExplorer) uicomp.getAncestorOfType(UIContentExplorer.class);      
      String uri = event.getParameter(OBJECTID);                      
      try {                                  
        if(uri.startsWith(uiExplorer.getRootPath())) {
          uiExplorer.setSelectNode(uri); 
        }        
      } catch (Exception e) {        
        Information iprovider = findInformationProvider(uicomp);
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }


  static public class ViewCategoryActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {            
      UIContentList uicomp= (UIContentList)event.getComponent() ;
      UIContentExplorer uiExplorer = 
        (UIContentExplorer) uicomp.getAncestorOfType(UIContentExplorer.class);
      String uri = event.getParameter(OBJECTID) ;                 
      try{
        if(uri.startsWith(uiExplorer.rootPath_)) {
          uiExplorer.setSelectNode(uri) ; 
        }        
      } catch(Exception e) {
        Information iprovider = findInformationProvider(uicomp);
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }

  static public class ViewDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentList uicomp= (UIContentList)event.getComponent() ;
      UIDetailDocument uidoc = (UIDetailDocument)uicomp.getSibling(UIDetailDocument.class);
      String uri = event.getParameter(OBJECTID) ;
      List<Document> documents = uicomp.getAllDocuments() ;             
      Node node = null ; 
      for(Document document:documents) {
        if(document.getUri().equals(uri)) {
          node = document.getNode() ;
        }
      }
      if(node != null){
        uidoc.setNode(node) ;
      } else {
        uidoc.setNodeByUri(uri) ;
      }
      UIContentExplorer uiExplorer = (UIContentExplorer)uicomp.getParent() ;
      uiExplorer.setShowingToolbar(false) ;
      uiExplorer.setShowingTagsCloud(false) ;      
      //uicomp.setRenderedSibling(UIDetailDocument.class) ;
      uicomp.setRendered(false) ;
      UIDetailDocument doc = (UIDetailDocument)uicomp.getSibling(UIDetailDocument.class) ;
      doc.setOpener(UIContentList.class) ;
      doc.setRendered(true) ;
      ExternalContext eContext = uicomp.getFacesContext().getExternalContext();
      ActionResponse response = (ActionResponse) eContext.getResponse();
      response.setWindowState(WindowState.MAXIMIZED);      
    }    
  }      	

  static public class RefreshListActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentList uicomp= (UIContentList)event.getComponent() ;     
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
      String usecase = prefs.getValue(CBUtils.USECASE, "") ;
      if(usecase.equals(CBUtils.USE_SCRIPT)) {
        String scriptName = prefs.getValue(CBUtils.SCRIPT_NAME, "") ;
        String wsName = prefs.getValue(CBUtils.WORKSPACE, "") ;
        UIContentBrowserPortlet cbportlet = 
          (UIContentBrowserPortlet)uicomp.getAncestorOfType(UIContentBrowserPortlet.class) ;
        uicomp.updateScriptData(cbportlet.getNodesByScript(scriptName), wsName) ;
      }else 
        uicomp.updateDocumentList();      
    }
  }


  protected class DocumentComparator implements Comparator<Document> {    
    private String oder_ ;

    public DocumentComparator(String oder) {
      this.oder_ = oder ;
    }    

    public int compare(Document node1, Document node2) {
      try{
        String name1 = node1.getName() ;
        String name2 = node2.getName() ;
        if(ASCENDING_ORDER.equals(oder_)) 
          return name1.compareToIgnoreCase(name2) ;
        return name2.compareToIgnoreCase(name1) ;
      }catch (Exception e) {
      }      
      return 0;
    }
  }
  
}
