package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.query.Query;
import javax.portlet.PortletPreferences;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.queries.QueryService;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;

public class UIQueryConfig extends UISimpleForm {	

  final public static String SQL_QUERY = "select * from exo:article where jcr:path like '/cms/publications/%'" ;
  final public static String XPATH_QUERY = "/jcr:root/cms/publications//element(*, exo:article)" ;
  final public static String CHANGE_QUERY_LANGUAGE_ACTION = "changeQueryLanguage" ;
  final public static String CHANGE_QUERY_TYPE_ACTION = "changeQueryType" ;
  final public static String CHANGE_CREATE_QUERY_ACTION = "changeCreateQuery" ;
  final public static String NEW_CONFIG_ACTION = "newConfig".intern() ;
  final public static String EDIT_CONFIG_ACTION = "editConfig".intern() ;
  final public static String NEW_QUERY = "newQuery" ;
  final public static String CREATE_QUERY = "createQuery" ;
  final public static String QUERY_TYPE = "queryType" ;
  final public static String PERSONAL = "personal" ;
  final public static String SHARED = "shared" ; 
  
  public static Parameter[] EDIT_CONFIG_PARAMS = { new Parameter(ACTION,EDIT_CONFIG_ACTION)} ;
  public static Parameter[] NEW_CONFIG_PARAMS = {new Parameter(ACTION,NEW_CONFIG_ACTION)} ;  


  private UISelectBox queryLanguage_;  
  private UISelectBox queryStore_;
  private UITextArea queryString_ ;
  private UIStringInput nbPerPage_ ;	
  private UISelectBox detailTemplatesBox_ ;  
  private UISelectBox queryTemplatesBox_ ;
  protected UISelectBox workspaces_ ;
  private UICheckBox allowVoting_ ;
  private UICheckBox allowComment_ ;
  private UICheckBox viewTagsCloud_ ;
  private boolean editable_ ;
  private boolean isChangedForm_ ;
  private boolean isCreateNewQuery_  = true ;
  private boolean isPersonalQuery_  = true ;
  private QueryService qservice_ ;
  private List roles_ = new ArrayList();

  public UIQueryConfig(QueryService qservice, OrganizationService oservice) throws Exception {		
    super("QueryConfig","post",null) ;
    setId("QueryConfig") ;
    qservice_ = qservice ;
    PortletPreferences prefs = FacesUtil.getPortletPreferences();    
    String[] workspaceNames = CBUtils.getWorkspaceNames() ;    
    List<SelectItem> workspaceOptions = new ArrayList<SelectItem>() ;    
    for(String workspace:workspaceNames) {
      workspaceOptions.add(new SelectItem(workspace,workspace)) ;
    } 
    
    isCreateNewQuery_ = Boolean.parseBoolean(prefs.getValue(CBUtils.IS_NEW_QUERY,"")) ;
    isPersonalQuery_ = Boolean.parseBoolean(prefs.getValue(CBUtils.IS_PERSONAL_QUERY,"")) ;
    
    workspaces_ = new  UISelectBox(CBUtils.WORKSPACE, prefs.getValue(CBUtils.WORKSPACE,""), workspaceOptions) ;
    add(workspaces_);
    
    queryTemplatesBox_ = new UISelectBox(CBUtils.TEMPLATE,"",new ArrayList<SelectItem>()) ;
    add(queryTemplatesBox_) ;
    
    List<SelectItem> options = new ArrayList<SelectItem>();    
    options.add(new SelectItem(Query.SQL, Query.SQL));
    options.add(new SelectItem(Query.XPATH, Query.XPATH));
    String language = prefs.getValue(CBUtils.QUERY_LANGUAGE, "") ;
    queryLanguage_ = new UISelectBox(CBUtils.QUERY_LANGUAGE, language, options);
    queryLanguage_.setUpdateOnChangeAction(CHANGE_QUERY_LANGUAGE_ACTION) ;
    queryLanguage_.setOptions(options) ;
    add(queryLanguage_);
    
    List<SelectItem> createOptions = new ArrayList<SelectItem>();    
    createOptions.add(new SelectItem("New query", NEW_QUERY));
    createOptions.add(new SelectItem("From existing queries", "existQuery"));
    UISelectBox createQuery = new UISelectBox(CREATE_QUERY,"",createOptions) ;
    createQuery.setUpdateOnChangeAction(CHANGE_CREATE_QUERY_ACTION) ;
    if(isCreateNewQuery_) createQuery.setValue(NEW_QUERY) ;
    else createQuery.setValue("existQuery") ;
    add(createQuery) ;
    
    List<SelectItem> queryOptions = new ArrayList<SelectItem>();    
    queryOptions.add(new SelectItem("Personal query", PERSONAL));
    queryOptions.add(new SelectItem("Shared query", SHARED));
    UISelectBox queryType = new UISelectBox(QUERY_TYPE,"",queryOptions) ;
    queryType.setUpdateOnChangeAction(CHANGE_QUERY_TYPE_ACTION) ;
    if(isPersonalQuery_) queryType.setValue(PERSONAL) ;
    else queryType.setValue(SHARED) ;    
    add(queryType) ;    
    FacesContext context  = FacesContext.getCurrentInstance() ;
    ExternalContext econtext = context.getExternalContext()  ;
    String userName = econtext.getRemoteUser() ;
    Collection memberships = oservice.getMembershipHandler().findMembershipsByUser(userName) ;
    if(memberships != null && memberships.size() > 0){
      Object[] objects = memberships.toArray() ;      
      for(int i = 0 ; i < objects.length ; i ++ ){
        Membership membership = (Membership)objects[i] ;
        String role = membership.getMembershipType() + ":" + membership.getGroupId() ;
        roles_.add(role) ;
      } 
    }
    roles_.add(userName) ;
    
    List<SelectItem> storeQueryoptions = new ArrayList<SelectItem>() ;
    if(isPersonalQuery_) {
      List<Query> queries = qservice_.getQueries();
      for(Query query : queries) {
        String path = query.getStoredQueryPath() ;
        if(query.getLanguage().equals(language))
          storeQueryoptions.add(new SelectItem(path.substring(path.lastIndexOf("/")+ 1), path)) ;
      }      
    }else {        
      List<Node> queries = qservice_.getSharedQueries(language, roles_);
      for(Node query : queries) {
        storeQueryoptions.add(new SelectItem(query.getName(), query.getPath())) ;
      }      
    }  
    queryStore_ = new UISelectBox(CBUtils.QUERY_STORE, prefs.getValue(CBUtils.QUERY_STORE,""), null);
    queryStore_.setOptions(storeQueryoptions) ;
    
    add(queryStore_);    
    
    queryString_ = new UITextArea( CBUtils.QUERY_STATEMENT,XPATH_QUERY) ;
    add(queryString_) ;   
    nbPerPage_ = new UIStringInput(CBUtils.NB_PER_PAGE,prefs.getValue(CBUtils.NB_PER_PAGE,"")) ;
    add(nbPerPage_) ;

    detailTemplatesBox_ = new UISelectBox(CBUtils.BOX_TEMPLATE,"",new ArrayList<SelectItem>()) ;
    add(detailTemplatesBox_) ;
    
    allowVoting_ = new UICheckBox(CBUtils.ALLOW_VOTING,prefs.getValue(CBUtils.ALLOW_VOTING,"")) ;
    add(allowVoting_) ;
    
    allowComment_ = new UICheckBox(CBUtils.ALLOW_COMMENT,prefs.getValue(CBUtils.ALLOW_COMMENT,"")) ;
    add(allowComment_) ;
    
    viewTagsCloud_ = new UICheckBox(CBUtils.VIEW_TAGS_CLOUD,prefs.getValue(CBUtils.VIEW_TOOLBAR,"")) ;
    add(viewTagsCloud_) ;
    
    updateSelectBoxes() ;
    
    setRendererType("SimpleFormVelocityRenderer");    
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
    addActionListener(ChangeQueryLanguageActionListener.class,CHANGE_QUERY_LANGUAGE_ACTION) ;
    addActionListener(ChangeCreateQueryActionListener.class,CHANGE_CREATE_QUERY_ACTION) ;
    addActionListener(ChangeQueryTypeActionListener.class,CHANGE_QUERY_TYPE_ACTION) ;
    addActionListener(NewConfigActionListener.class,NEW_CONFIG_ACTION) ;
    addActionListener(EditConfigActionListener.class,EDIT_CONFIG_ACTION) ;    
  }	

  public boolean isEditable() {return editable_ ; }
  public boolean isCreateNewQuery() { return isCreateNewQuery_ ;  }
  public boolean isPersonalQuery() {return isPersonalQuery_ ; }
  
  public String getEditConfigParams() { return EDIT_CONFIG_ACTION ;}
  public String getNewConfigParams() { return NEW_CONFIG_ACTION ;}  

  public void updateFormFields() throws Exception{      
    PortletPreferences prefs = FacesUtil.getPortletPreferences();
    queryLanguage_.setValue(prefs.getValue(CBUtils.QUERY_LANGUAGE,"")) ;    
    queryString_.setValue(prefs.getValue(CBUtils.QUERY_STATEMENT,"")) ;
    queryTemplatesBox_.setValue(prefs.getValue(CBUtils.TEMPLATE,"")) ;
    workspaces_.setValue(prefs.getValue(CBUtils.WORKSPACE,"")) ;
    nbPerPage_.setValue(prefs.getValue(CBUtils.NB_PER_PAGE,"")) ;  
    detailTemplatesBox_.setValue(prefs.getValue(CBUtils.BOX_TEMPLATE,"")) ;
    viewTagsCloud_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.VIEW_TAGS_CLOUD,""))) ;
    allowComment_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_COMMENT,""))) ;
    allowVoting_.setSelect(Boolean.parseBoolean(prefs.getValue(CBUtils.ALLOW_VOTING,""))) ;    
  }

  public void updateSelectBoxes() throws Exception {
    ManageViewService viewService = 
      (ManageViewService)PortalContainer.getComponent(ManageViewService.class) ;    
    List<SelectItem> queryTempOptions = new ArrayList<SelectItem>() ;
    List<Node> querylTemplates = viewService.getAllTemplates(BasePath.CB_QUERY_TEMPLATES) ;
    for(Node template:querylTemplates) {
      queryTempOptions.add(new SelectItem(template.getName(),template.getName())) ;
    }
    queryTemplatesBox_.setOptions(queryTempOptions) ;

    List<SelectItem> docTempOptions = new ArrayList<SelectItem>() ;
    List<Node> docTemplates = viewService.getAllTemplates(BasePath.CB_DETAIL_VIEW_TEMPLATES) ;
    for(Node template: docTemplates) {
      docTempOptions.add(new SelectItem(template.getName(),template.getName())) ;
    }
    detailTemplatesBox_.setOptions(docTempOptions) ;
  }
  public void setEditable(boolean editable) {
    editable_ = editable ;
    queryLanguage_.setEditable(editable_);  
    queryString_.setEditable(editable_) ;
    nbPerPage_.setEditable(editable_) ;  
    detailTemplatesBox_.setEditable(editable_) ;    
    queryTemplatesBox_.setEditable(editable_) ;
    workspaces_.setEditable(editable_);
    queryStore_.setEditable(editable_) ;
    getUIInput(CREATE_QUERY).setEditable(editable_) ;
    getUIInput(QUERY_TYPE).setEditable(editable_) ;
    viewTagsCloud_.setEditable(editable_) ;
    allowComment_.setEditable(editable_) ;
    allowVoting_.setEditable(editable_) ;
  }

  public void setChangedForm(boolean isChangedForm) {
    isChangedForm_ = isChangedForm ;
  }

  public static class NewConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryConfig queryConfig = (UIQueryConfig)event.getSource() ;      
      queryConfig.setRenderedSibling(UINewConfigForm.class) ;
    }    
  }      

  public static class EditConfigActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryConfig queryConfig = (UIQueryConfig)event.getSource() ;
      queryConfig.setEditable(true) ;      
      queryConfig.setChangedForm(false) ;
    }    
  }  
 
  public static class ChangeQueryLanguageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryConfig queryConfig= (UIQueryConfig)event.getSource() ;
      String queryLanguage = queryConfig.queryLanguage_.getValue() ;
      if(Query.XPATH.equals(queryLanguage)) {
        queryConfig.queryString_.setText(XPATH_QUERY) ;
      }else {
        queryConfig.queryString_.setText(SQL_QUERY) ;
      }
      String queryType = queryConfig.getUISelectBox(QUERY_TYPE).getValue() ;
      if(queryType.equals(PERSONAL)) {
        queryConfig.getUISelectBox(QUERY_TYPE).setValue(PERSONAL) ;
        List<SelectItem> options = new ArrayList<SelectItem>() ;
        List<Query> queries = queryConfig.qservice_.getQueries();
        for(Query query : queries) {
          String path = query.getStoredQueryPath() ;
          if(query.getLanguage().equals(queryLanguage))
            options.add(new SelectItem(path.substring(path.lastIndexOf("/")+ 1), path)) ;
        }
        queryConfig.queryStore_.setOptions(options) ;
      }else {
        List<SelectItem> options = new ArrayList<SelectItem>() ;
        List<Node> queries = queryConfig.qservice_.getSharedQueries(queryLanguage, queryConfig.roles_);
        for(Node query : queries) {
          options.add(new SelectItem(query.getName(), query.getPath())) ;
        }
        queryConfig.queryStore_.setOptions(options) ;
      }      
    }    
  }
  
  public static class ChangeCreateQueryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryConfig queryConfig = (UIQueryConfig)event.getSource() ;
      String createType = queryConfig.getUISelectBox(CREATE_QUERY).getValue() ;
      if(createType.equals(NEW_QUERY)) {
        queryConfig.isCreateNewQuery_ = true ;
      }else {
        queryConfig.isCreateNewQuery_ = false ;
        String queryLanguage = queryConfig.getUISelectBox(CBUtils.QUERY_LANGUAGE).getValue() ;
        if(queryConfig.isPersonalQuery_){
          queryConfig.getUISelectBox(QUERY_TYPE).setValue(PERSONAL) ;
        }else {
          queryConfig.getUISelectBox(QUERY_TYPE).setValue(SHARED) ;
        }        
        List<SelectItem> options = new ArrayList<SelectItem>() ;
        List<Query> queries = queryConfig.qservice_.getQueries();
        for(Query query : queries) {
          String path = query.getStoredQueryPath() ;
          if(query.getLanguage().equals(queryLanguage))
            options.add(new SelectItem(path.substring(path.lastIndexOf("/")+ 1), path)) ;
        }
        queryConfig.queryStore_.setOptions(options) ;
      }      
    }    
  } 
  
  public static class ChangeQueryTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryConfig queryConfig = (UIQueryConfig)event.getSource() ;
      String createType = queryConfig.getUISelectBox(QUERY_TYPE).getValue() ;
      String queryLanguage = queryConfig.getUISelectBox(CBUtils.QUERY_LANGUAGE).getValue() ;
      if(createType.equals(PERSONAL)) {
        queryConfig.getUISelectBox(QUERY_TYPE).setValue(PERSONAL) ;
        List<SelectItem> options = new ArrayList<SelectItem>() ;
        List<Query> queries = queryConfig.qservice_.getQueries();
        for(Query query : queries) {
          String path = query.getStoredQueryPath() ;
          if(query.getLanguage().equals(queryLanguage))
            options.add(new SelectItem(path.substring(path.lastIndexOf("/")+ 1), path)) ;
        }
        queryConfig.queryStore_.setOptions(options) ;
      }else {        
        List<SelectItem> options = new ArrayList<SelectItem>() ;
        List<Node> queries = queryConfig.qservice_.getSharedQueries(queryLanguage, queryConfig.roles_);
        for(Node query : queries) {
          options.add(new SelectItem(query.getName(), query.getPath())) ;
        }
        queryConfig.queryStore_.setOptions(options) ;
      }      
    }
  }
  
  /*public static class ChangeSharedQueryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryConfig queryConfig = (UIQueryConfig)event.getSource() ;
      String queryPath = queryConfig.sharedQueries_.getValue() ;
      if(queryPath != "" && queryPath.length() > 0) {
        Query query = queryConfig.qservice_.getQuery(queryPath) ;
        queryConfig.queryString_.setValue(query.getStatement()) ;
      }      
    }    
  }*/

  public static class CancelActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception { 
      UIQueryConfig queryConfig= (UIQueryConfig)event.getSource() ;
      if(queryConfig.isChangedForm_) {        
        queryConfig.setRenderedSibling(UINewConfigForm.class) ;
      }else {
        queryConfig.setEditable(false) ;        
      }
    }    
  }

  public static class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryConfig queryConfig= (UIQueryConfig)event.getSource() ;
      Information iprovider = findInformationProvider(queryConfig);     
      ResourceBundle res = getApplicationResourceBundle() ;
      PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
      String isCreateNew = queryConfig.getUIInput(CREATE_QUERY).getValue() ;
      String workspace = queryConfig.getUIInput(CBUtils.WORKSPACE).getValue() ;
      String queryLanguage = queryConfig.getUISelectBox(CBUtils.QUERY_LANGUAGE).getValue() ;
      String queryStatement = queryConfig.getUIInput(CBUtils.QUERY_STATEMENT).getValue() ;      
      String queryStore = "" ;
      if(!isCreateNew.equals(NEW_QUERY)) {
        queryStore = queryConfig.getUIInput(CBUtils.QUERY_STORE).getValue() ;
        if(queryStore == null || queryStore.length() < 1) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
              res.getString("UIContentBrowserConfig.msg.there-are-no-query-selected"))) ;
          return ;
        }
      }
      String queryTemplate = queryConfig.getUIInput(CBUtils.TEMPLATE).getValue() ;
      String nbPerPage = queryConfig.getUIInput(CBUtils.NB_PER_PAGE).getValue() ;
      String boxTemplate = queryConfig.getUISelectBox(CBUtils.BOX_TEMPLATE).getValue() ;
      String viewToolbar = prefs.getValue(CBUtils.VIEW_TOOLBAR,"") ;      
      String path = prefs.getValue(CBUtils.JCR_PATH,"") ;
      String showRefDocument = prefs.getValue(CBUtils.REF_DOCUMENT,"") ;
      String showChildDocument = prefs.getValue(CBUtils.CHILD_DOCUMENT,"") ;
      String isNewQuery = String.valueOf(queryConfig.isCreateNewQuery_) ;
      String isPersonalQuery = String.valueOf(queryConfig.isPersonalQuery_) ;      
      UICheckBox allowVotingBox = (UICheckBox)queryConfig.getUIInput(CBUtils.ALLOW_VOTING) ;
      UICheckBox allowCommentBox = (UICheckBox)queryConfig.getUIInput(CBUtils.ALLOW_COMMENT) ;
      String allowVoting = Boolean.toString(allowVotingBox.getSelect()) ;
      String allowComment = Boolean.toString(allowCommentBox.getSelect());   
      UICheckBox viewTagsCloudBox = (UICheckBox)queryConfig.getUIInput(CBUtils.VIEW_TAGS_CLOUD);
      String viewTagsCloud = Boolean.toString(viewTagsCloudBox.getSelect()) ;
      
      String usecase = CBUtils.USE_JCR_QUERY ;
      String documentName = prefs.getValue(CBUtils.DOCUMENT_NAME,"") ;
      if(documentName == null ||documentName.length() == 0) documentName = "" ;      
      UIContentBrowserConfig uiconfig = (UIContentBrowserConfig)queryConfig.getParent() ;			
      uiconfig.saveBrowserConfig(workspace, path, queryLanguage, queryStatement, isNewQuery, 
          isPersonalQuery, queryStore, nbPerPage, queryTemplate, boxTemplate, showRefDocument, 
          showChildDocument, viewToolbar, usecase, documentName, "", viewTagsCloud, 
          allowVoting, allowComment) ;
      iprovider.addMessage(
          new ExoFacesMessage(FacesMessage.SEVERITY_INFO,res.getString("UIContentBrowserConfig.msg.save-config-successfully"))) ;
      queryConfig.updateFormFields() ;
      queryConfig.setEditable(false) ;
    }
  }
}
