package org.exoplatform.portlets.content.drives.component;

import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import javax.jcr.Node;
import javax.jcr.Session;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.JCRUtil;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.cms.drives.DriveData;
import org.exoplatform.services.cms.drives.ManageDriveService;
import org.exoplatform.services.jcr.RepositoryService;

public class UIDriveList extends UIExoCommand {
  
  final static public String ADD_ACTION = "add".intern(); 
  final static public String TEMPLATE_ACTION = "template".intern();
  
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
  
  private ManageDriveService dservice_ ;
  public UIPageListIterator uiPageIterator_ ;  
  
  public UIDriveList(ManageDriveService dservice) throws Exception{    
    setId("UIDriveList") ;
    setRendererType("VelocityRenderer") ;
    dservice_ = dservice ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    uiPageIterator_.setPageList(new ContentPageList(getAllDrives()));  
  }  
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  
  public List getAllDrives() throws Exception{
    return dservice_.getAllDrives() ;
  }
  
  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getAllDrives()));        
  }
  
  public String getIcon(DriveData drive) throws Exception {
    String icon = null ;
    String path = drive.getIcon() ;
    if(path != null && path.length() > 0) {      
      try{
        RepositoryService repositoryService = 
          (RepositoryService)PortalContainer.getInstance().getComponentInstanceOfType(RepositoryService.class) ;
        Session session = repositoryService.getRepository().getSystemSession("digital-assets") ;
        Node file = (Node)session.getItem(path) ;
        if(file.hasNode("jcr:content")){
          String portalName = PortalContainer.getInstance().getPortalContainerInfo().getContainerName() ;
          icon = "<img src=\"/"+portalName+"/jcr?workspace=digital-assets&path=" + file.getPath() +
          "&relativePath=jcr:content\" width=\"16\" height=\"16\" />" ;
          return icon ;
        }
      }catch (Exception ex) {        
      }      
    }
    ResourceBundle res = JCRUtil.getPortletBundle();
    icon = res.getString("UIWorkspaceBrowser.icon.workspace." + drive.getWorkspace());
    return icon;
  }
  
  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveList driveList = (UIDriveList)event.getSource() ;
      UIDriveForm drive = (UIDriveForm)driveList.getSibling(UIDriveForm.class) ;
      drive.refresh(null) ;
      driveList.setRenderedSibling(UIDriveForm.class) ;
    }
  }
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try{
        UIDriveList driveList = (UIDriveList)event.getSource() ;
        String driveName = event.getParameter(OBJECTID) ;
        UIDriveForm drive = (UIDriveForm)driveList.getSibling(UIDriveForm.class) ;
        drive.refresh(driveName) ;
        driveList.setRenderedSibling(UIDriveForm.class) ;
      }catch(Exception e) {
        e.printStackTrace() ;
      }      
    }
  }
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try{
        UIDriveList driveList = (UIDriveList)event.getSource() ;       
        String driveName = event.getParameter(OBJECTID) ;
        driveList.dservice_.removeDrive(driveName) ;
        driveList.refresh();
      }catch(Exception e){
        e.printStackTrace() ;
      }      
    }
  }
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveList driveList = (UIDriveList)event.getSource() ;
      driveList.setRenderedSibling(UIECMAdminSummary.class) ;
    }
  }
}
