/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.folksonomy.component;

import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.folksonomy.FolksonomyService;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Xuan Hoa
 *          hoa.pham@exoplatform.com
 * Dec 28, 2006  
 */
public class UITagStyleForm extends UISimpleForm {

  final static String STYLE_NAME = "styleName".intern() ;
  final static String RANGE = "range".intern() ;
  final static String HTML_STYLE = "htmlStyle".intern() ;
  final static String RANGE_PROP = "exo:styleRange".intern() ;
  final static String HTML_STYLE_PROP = "exo:htmlStyle".intern() ;

  private Node selectedTagStyle_ ;  
  private UIStringInput nameInut_ ; 
  private UIStringInput rangeInput_ ;
  private UITextArea htmlStyleInut_ ;
  
  public UITagStyleForm() throws Exception {
    super("UITagStyleForm", "post",null);
    setRendererType("SimpleFormVelocityRenderer");
    
    nameInut_ = new UIStringInput(STYLE_NAME,"") ;    
    rangeInput_ = new UIStringInput(RANGE,"") ;
    htmlStyleInut_ = new UITextArea(HTML_STYLE,"") ;
    nameInut_.setReadonly(true) ;
    add(nameInut_) ;
    add(rangeInput_) ;
    add(htmlStyleInut_) ;

    addActionListener(UpdateActionListener.class,SAVE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
  }


  public Node getTagStyle() { return selectedTagStyle_ ; }
  
  public void setTagStyle(Node selectedTagStyle) throws Exception { 
    this.selectedTagStyle_ = selectedTagStyle ;      
    this.getUIStringInput(STYLE_NAME).setText(selectedTagStyle_.getName()) ;    
    String range = selectedTagStyle_.getProperty(RANGE_PROP).getValue().getString() ;    
    rangeInput_.setValue(range) ;
    String htmlStyle = selectedTagStyle_.getProperty(HTML_STYLE_PROP).getValue().getString() ;
    htmlStyleInut_.setValue(htmlStyle) ;
  }

  public static class UpdateActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {    
      UITagStyleForm styleForm = (UITagStyleForm)event.getComponent() ;
      Node selectedNode = styleForm.getTagStyle() ;
      FolksonomyService folkService = 
        (FolksonomyService)PortalContainer.getComponent(FolksonomyService.class) ;
      String range = styleForm.getUIStringInput(UITagStyleForm.RANGE).getValue() ;
      String htmlStyle = styleForm.getUIInput(UITagStyleForm.HTML_STYLE).getValue() ;
      Information iprovider = findInformationProvider(styleForm) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      if(!validateRange(range)) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UITagStyleForm.msg.range-validator"))) ;
        return ;
      }try{
        folkService.updateStype(selectedNode.getPath(),range,htmlStyle) ; 
      }catch (Exception e) {
        //e.printStackTrace() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,e.getMessage())) ;
      }        
      iprovider.addMessage(
          new ExoFacesMessage(FacesMessage.SEVERITY_INFO,res.getString("UITagStyleForm.msg.update-succesfully"))) ;
      styleForm.setRenderedSibling(UITagStyleList.class) ;
    }    

    private boolean validateRange(String range) {      
      String[] vars = null ;      
      try{
        vars = StringUtils.split(range,"..") ;
      }catch (Exception e) {  
        e.printStackTrace() ;
        return false ;        
      }                  
      if(vars == null || vars.length!= 2) return false ;
      String minRange = vars[0], maxRange = vars[1] ;
      if(!StringUtils.isNumeric(minRange)) return false ;      
      try{
        int min = Integer.parseInt(vars[0]) ;
        if(min<0) return false ;      
        if(!StringUtils.isNumeric(maxRange)) {
          if(!maxRange.equals("*")) return false ;
        }else{
          if(Integer.parseInt(maxRange)<=0) return false ;
        }
      }catch (Exception e) {    
        return false ;
      }
      return true ;
    }
  }

  public static class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITagStyleForm styleForm = (UITagStyleForm)event.getComponent() ;
      styleForm.setRenderedSibling(UITagStyleList.class) ;
    }    
  }

}
