/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.folksonomy.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.cms.folksonomy.FolksonomyService;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Xuan Hoa
 *          hoa.pham@exoplatform.com
 * Dec 27, 2006  
 */
public class UITagStyleList extends UIExoCommand {
  final static String RANGE_PROP = "exo:styleRange".intern() ;
  final static String HTML_STYLE_PROP = "exo:htmlStyle".intern() ;
  private FolksonomyService folksonomyService_ ;
  private List<Node> styleList_  = new ArrayList<Node>();
  
  static Parameter[] EDIT_PARAMS = { new Parameter(ACTION,EDIT_ACTION)} ;
  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION,CANCEL_ACTION)} ;
  
  public UITagStyleList(FolksonomyService folksonomyService) throws Exception {
    setId("UITagStyleList") ;
    setRendererType("VelocityRenderer") ;
    folksonomyService_ = folksonomyService ;
    
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
    addActionListener(EditStyleActionListener.class,EDIT_ACTION) ;
    addChild(UITagStyleForm.class).setRendered(false) ;
  }
  
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getCancelParams() { return  CANCEL_PARAMS ;} 
  
  public List<Node>getTagStyleList() throws Exception{    
    styleList_ = folksonomyService_.getAllTagStyle() ;
    return styleList_ ;
  }
  
  public String getRangeOfStyle(Node tagStyle) throws Exception {
    String range = tagStyle.getProperty(RANGE_PROP).getValue().getString() ;
    return range ;
  }
  
  public String getHtmlStyleOfStyle(Node tagStyle) throws Exception {
    String htmlStyle = tagStyle.getProperty(HTML_STYLE_PROP).getValue().getString() ;
    return htmlStyle ;
  }  
  
  public Node getSelectedTagStyle(String tagStyleName) throws Exception {
    for(Node tagStyle: styleList_) {
      if(tagStyle.getName().equals(tagStyleName)) return tagStyle ;
    }
    return null ;
  }
  
  public static class EditStyleActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITagStyleList tagStyleList = (UITagStyleList)event.getComponent() ;
      String selectedName = event.getParameter(OBJECTID) ;
      Node selectedTagStyle = tagStyleList.getSelectedTagStyle(selectedName) ;
      if(selectedTagStyle == null) {        
        return ;
      }
      UITagStyleForm styleForm = 
        (UITagStyleForm)tagStyleList.getSibling(UITagStyleForm.class) ;
      styleForm.setTagStyle(selectedTagStyle) ;
      tagStyleList.setRenderedSibling(UITagStyleForm.class) ;
    }    
  }  
  
  
  public static class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITagStyleList tagStyleList = (UITagStyleList)event.getComponent() ;      
      UIFolksonomyManager folksonomyManager = (UIFolksonomyManager)tagStyleList.getParent() ;
      folksonomyManager.setRenderedSibling(UIECMAdminSummary.class) ;
    }    
  }
  
}
