package org.exoplatform.portlets.content.jcr;
import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.servlet.VelocityServlet;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.faces.cms.CMSViewComponent;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.templates.velocity.impl.JCRResourceLoaderImpl;

public class ViewContentServlet extends VelocityServlet{   
  public Template handleRequest( HttpServletRequest request, 
                                   HttpServletResponse response, Context context ) {
    String portalName = (String)request.getAttribute("portalName");
    String templateName = (String)request.getAttribute("templateName");
    String requestURL = request.getRequestURL().toString() ;
    String baseURL = requestURL.substring(0,requestURL.indexOf("/rss") + 4) ;
    String wsName = (String)request.getAttribute("wsName");
    Node node = (Node)request.getAttribute("curNode");
    Template template = null;
    try {
      PortalContainer pcontainer = getPortalContainer(portalName) ;
      PortalContainer.setInstance(pcontainer) ;
      JCRResourceLoaderImpl jcrResourceLoader = (JCRResourceLoaderImpl)pcontainer
        .getComponentInstanceOfType(JCRResourceLoaderImpl.class) ;
      RepositoryService repositoryService  = 
        (RepositoryService)pcontainer.getComponentInstanceOfType(RepositoryService.class) ;
      Session session = repositoryService.getRepository().getSystemSession(wsName) ;
      AdaptUIComponent uicomponent = new AdaptUIComponent(session, node, portalName) ;
      uicomponent.setNode(node) ;
      uicomponent.setRssLink(baseURL + "/" + portalName + "/") ;
      context.put("uicomponent", uicomponent);
      template = (Template)jcrResourceLoader.getResource(templateName, 1, "UTF-8") ;          
    } catch( Exception e ) {
      e.printStackTrace() ;
    }
    return template;
  }
  
  private  PortalContainer getPortalContainer(String portalName) {
    PortalContainer pcontainer =  RootContainer.getInstance().getPortalContainer(portalName) ;
    return pcontainer ;
  }
  
  public class AdaptUIComponent implements CMSViewComponent{
    
    private Node node_ ;
    private String rssLink_ ;
    private Session session_ ;
    private String portalName_ ;
    
    public AdaptUIComponent(Session session, Node node, String portalName) {
      node_ = node ;
      session_ = session ;
      portalName_ = portalName ;
    }
    
    public void setNode(Node curNode) { node_ = curNode ; }    
    public Node getNode() { return node_ ; }
    
    public boolean isRssLink() { return true ; }
    
    public String getRssLink(){ return rssLink_ ; }
    public void setRssLink(String link) { rssLink_ = link ; }
    
    public String getPortalName() { return portalName_ ; }
    public String getWorkspaceName() { return session_.getWorkspace().getName() ; }
    
    public List getAttachments() throws Exception{ 
      List<Node> attachments = new ArrayList<Node>() ;
      String nodeType = "";
      NodeIterator childrenIterator;
      childrenIterator = node_.getNodes();
      while (childrenIterator.hasNext()) {
        Node childNode = childrenIterator.nextNode();
        try {
          nodeType = childNode.getPrimaryNodeType().getName();
          if ("nt:file".equals(nodeType))
            attachments.add(childNode);
        } catch (Exception e) {}
      }
      return attachments;
    }
    
    public List getRelations() throws Exception{ 
      List<Node> relations = new ArrayList<Node>() ;
      try {
        Value[] vals = node_.getProperty("exo:relation").getValues();
        for (int i = 0; i < vals.length; i++) {
          String uuid = vals[i].getString();
          Node node = session_.getNodeByUUID(uuid);
          relations.add(node);
        }
      } catch (Exception e) {}
      return relations;
    }
    
    public boolean hasPropertyContent(Node node, String property){
      try {
        String value = node.getProperty(property).getString() ;
        if(value.length() > 0) return true ;
      }catch (Exception e) {
        e.printStackTrace() ;      
      }
      return false ;
    }
    
    public String getNodeType() throws Exception {
      return null;
    }

    public String getTemplatePath() throws Exception {
      return null;
    }
    
    public Object getComponentInstanceOfType(String className) {
      return null;
    }    

    public WikiContext getWikiContext() {
      return null;
    }

    public WikiEngineService getWikiService() {
      return null;
    }

    public boolean isNodeTypeSupported() {
      return false;
    }
    
    public String getAttachmentSize(Node node){
      return null ; 
    }
    
    public List getSupportedLocalise() throws Exception {
      List local = new ArrayList() ;
      return local ;   
    }

    public String getLargeNodeTypeIcon(Node arg0) {
      return null;
    }

    public String getPropertyIconByName(String arg0) {
      return null;
    }

    public String getPropertyTypeIcon(Property arg0) {
      return null;
    }

    public String getSmallNodeTypeIcon(Node arg0) {
      return null;
    }
  }
}
