/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.io.InputStream;
import java.security.AccessControlException;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.AccessDeniedException;
import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIContentDisplayer.java,v 1.2 2004/08/07 18:11:26 tuan08 Exp $
 */
public class UIDocumentForm extends UISimpleMulltipartForm implements
    CMSDialogComponent {

  String documentType_;
  String documentName_;
  protected TemplateService templateService_;
  protected CmsService cmsService_;
  protected UIStringInput nameInput_;
  protected Node node_;
  protected ActionServiceContainer asContainer_ ;
  private InputStream binaryData_ ;
  private boolean isAddNew_ = true ;
  private boolean isReferenced_ ;

  public UIDocumentForm(CmsService cmsService, TemplateService templateService, 
      ActionServiceContainer asContainer)
      throws Exception {
    super("documentForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    nameInput_ = new UIStringInput("docName", "");
    cmsService_ = cmsService;
    templateService_ = templateService;
    asContainer_ = asContainer ;
    addActionListener(StoreDocumentActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  public void editDocument(Node editNode, boolean isReferenced) throws Exception {
    clean();
    isAddNew_ = false ;
    isReferenced_ = isReferenced ;    
    node_ = editNode ;
    nameInput_.setEditable(false) ;
    nameInput_.setValue(editNode.getName()) ;    
    documentType_ = editNode.getProperty("jcr:primaryType").getString();
    if("nt:file".equals(documentType_)) {
      Node jcrContent = editNode.getNode("jcr:content") ;      
      String mimeType = jcrContent.getProperty("jcr:mimeType").getString() ;
      if(!mimeType.startsWith("text")) {      
        setBinaryData(jcrContent.getProperty("jcr:data").getStream()) ;
      }
    }
  }

  public void showNewDocument(String documentType) throws Exception {
    clean();
    isAddNew_ = true ;    
    UIInput input = getUIInput("docName");
    input.setEditable(true);
    node_ = null;
    documentType_ = documentType;
  }
  
  public boolean isCheckedOut() throws RepositoryException {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Node node = uiExplorer.getSelectNode();
    
    return node.isCheckedOut();
  }

  public String getDocumentType() {
    return documentType_;
  }

  public String getDialogPath() throws Exception {
    return templateService_.getTemplatePath(true, documentType_);
  }

  public Node getNode() throws Exception {
    return node_;
  }
  
  public boolean isEditing() { return !isAddNew_ ; }
  
  public InputStream getBinaryData() { return binaryData_ ;}
  public void setBinaryData(InputStream data) { this.binaryData_ = data ; }
  
  public boolean isDocumentTypeSupported() throws Exception {
    return templateService_.isManagedNodeType(documentType_);
  }

  public void clean() {
    clearInputs();
    getChildren().clear();
    nameInput_.setValue("");
    add(nameInput_);
  }

  static public class StoreDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;      
      UIDocumentForm uiForm = (UIDocumentForm) event.getSource();
      
      String name = uiForm.nameInput_.getValue().trim();
      Information iprovider = findInformationProvider(uiForm);
      if (name == null || name.length() == 0) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
            res.getString("UIDocumentForm.msg.name-is-null")));
        return;
      }
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      Node storeHomeNode = uiExplorer.getSelectNode();
      if(!uiForm.isAddNew_) {
        if(uiForm.isReferenced_)storeHomeNode = uiForm.node_.getParent();
      }            
      Map sortedInputs = CMSUtils.prepareMap(uiForm);
      String nodePath = "/node" ;
      if(sortedInputs.containsKey(nodePath)) {
        JcrInputProperty property = (JcrInputProperty)sortedInputs.get(nodePath);            
        property.setValue(name);
        sortedInputs.put(nodePath, property);
      }else {
        JcrInputProperty property = new JcrInputProperty();
        property.setJcrPath(nodePath) ;
        property.setValue(name) ;
        sortedInputs.put(nodePath,property) ;
      }      
      if(uiForm.isEditing()) {
        if("nt:file".equals(uiForm.documentType_)) {
          JcrInputProperty jcrDataInput = new JcrInputProperty() ;
          jcrDataInput.setJcrPath("/node/jcr:content/jcr:data") ;
          jcrDataInput.setValue(uiForm.getBinaryData()) ;
          sortedInputs.put("/node/jcr:content/jcr:data",jcrDataInput) ;
        }
      }
      try {
        try {
          storeHomeNode.getSession().checkPermission(storeHomeNode.getPath(),"add_node,set_property");
        } catch (AccessControlException e) {
          throw new AccessDeniedException(e.getMessage());
        }
        String path = uiForm.cmsService_.storeNode(uiForm.documentType_, storeHomeNode, sortedInputs, uiForm.isAddNew_);
        storeHomeNode.save() ;
        String newNodePath = path.substring(path.indexOf(":") + 1 ) ;
        Node newNode = (Node)uiExplorer.getSession().getItem(newNodePath) ;
        if(!newNode.isNodeType("mix:i18n")) {
          newNode.addMixin("mix:i18n") ;          
        } 
        if(uiForm.isReferenced_){
          storeHomeNode.getSession().save() ;
          uiExplorer.broadcastOnChange(uiExplorer.getSelectNode());
        }else {
          event.addParameter("jcrPath", path);
          uiExplorer.broadcastOnChange(storeHomeNode);
        }
        if(!uiExplorer.getPreference().isJcrEnable()) uiExplorer.getSession().save() ;
      } catch (Exception e) {
    	  e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentForm uiForm = (UIDocumentForm) event.getComponent() ;
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode();
      uiForm.removeTransientActionListenerInterceptors();
      uiExplorer.broadcastOnChange(node);      
    }
  }
 
}