package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;

import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;


public class UIFolderForm extends UISimpleForm implements ChangeNodeListener {

  final static public String INPUT_NAME = "name";
  final static public String NT_UNSTRUCTURED = "nt:unstructured".intern() ;
  final static public String NT_FOLDER = "nt:folder".intern() ;  
  final static public String TYPE = "type".intern() ;
  
  protected Node node_;
  protected UISelectBox typeInput_;
  protected UIStringInput nameInput_ ;

  public UIFolderForm() throws Exception {
    super("folderForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    
    List<SelectItem> options = new ArrayList<SelectItem>();
    options.add(new SelectItem(NT_UNSTRUCTURED,NT_UNSTRUCTURED)) ;
    options.add(new SelectItem(NT_FOLDER,NT_FOLDER)) ;    
    typeInput_ = new UISelectBox(TYPE,NT_UNSTRUCTURED,options) ;
    add(typeInput_) ;
    
    nameInput_ = new UIStringInput("name","") ;
    add(nameInput_);

    addActionListener(AddFolderActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public void reset() {
  	nameInput_.setText("").setEditable(true) ;
  }
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false);
    node_ = node;
  }

  public Node getNode() {
    return node_;
  }

  static public class AddFolderActionListener extends ExoActionListener {

    public void execute(ExoActionEvent event) throws Exception {
      UIFolderForm uicomp = (UIFolderForm) event.getSource();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      String type = uicomp.typeInput_.getValue() ;
      String name = uicomp.getUIInput(INPUT_NAME).getValue().trim();      
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = uiExplorer.getSelectNode();        
        node.addNode(name, type);
        if(!uiExplorer.getPreference().isJcrEnable()) uiExplorer.getSession().save() ;
        uiExplorer.broadcastOnChange(node);
      } catch (Exception e) {        
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }  
}
