/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.Session;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIDateTimeInput;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.EmailAddressValidator;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.component.model.TicketCommandImpl;
import org.exoplatform.services.portletcontainer.ExoPortletRequest;
import org.exoplatform.services.ticket.TicketCommand;
import org.exoplatform.services.ticket.TicketService;


/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          nguyenchung136@yahoo.com
 * Jan 18, 2006
 */
public class UITicketForm extends UISimpleMulltipartForm implements ChangeNodeListener {

  private Node node_;
  protected Session session_ = null ;
  private static String UPDATE = "update", SENDMAIL="sendmail"; 
  
  static Parameter[] UPDATE_PARAMS   = { new Parameter(ACTION, UPDATE)};
  
  private UIStringInput user_,permission_,creator_,limit_,desc_,id_,mail_ ;
  private UIDateTimeInput toDate_;
  protected TicketService service_;
  private static String portalName_ ;
  protected boolean create=false,update=false;
  private GregorianCalendar calendar;
  private String creator;
  private String ticketId="";
  
  public UITicketForm() {
    super("createForm","post",null);
    setRendererType("SimpleFormVelocityRenderer");
    PortalContainer pcontainer =  PortalContainer.getInstance() ;
    service_ = (TicketService)pcontainer.getComponentInstanceOfType(TicketService.class) ;
    portalName_ =  pcontainer.getPortalContainerInfo().getContainerName() ;
    creator=pcontainer.getSessionManager().getLiveSessions().get(0).getRemoteUser() ;
    //  ---------------- Time-------------------------
    calendar = new GregorianCalendar() ;
    
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 0) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    
    user_ = new UIStringInput("users","") ;
    
    permission_=new UIStringInput("permission","");
    creator_=new UIStringInput("creator",creator);
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 24) ;
    toDate_=new UIDateTimeInput("accesstime",calendar.getTime());
    limit_=new UIStringInput("limit","-1");
    desc_=new UIStringInput("description","");
    id_=new UIStringInput("idticket","");
    id_.setEditable(false) ;
    mail_=new UIStringInput("sendmail","").addValidator(EmailAddressValidator.class) ;
    //------------------------- add -----------------
    reset();
    
    add(user_) ;
    add(permission_);
    add(creator_);
    add(toDate_);
    add(limit_);
    add(desc_);
    add(mail_);
    add(id_);
    
    addActionListener(CreateActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(UpdateActionListerner.class, UPDATE);    
  
  }
  
  public String getUpdateParams() { return UPDATE ; }
                     
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_=node;
    session_ = uiExplorer.getSession() ;
  }
  public Node getNode() {
    return node_;
  }
  /**
   * Method is reset defaul value in Form.
   */
  public void reset(){
   limit_.setText("-1");
   desc_.setText("");
   mail_.setText("");
   permission_.setText("");
   id_.setText("");
   user_.setText("");
   toDate_.addTime(calendar.getTime());
   creator_.setText(creator);
   create=false;
   update=true;
   
   limit_.setEditable(true);
   creator_.setEditable(true);
   desc_.setEditable(true);
   permission_.setEditable(true);
   user_.setEditable(true);
   mail_.setEditable(true);
   toDate_.setEditable(true);
   create=false;
   update=true;
   
  }
  
  static public class CreateActionListener extends ExoActionListener{
    private UITicketForm uiForm;    
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;      
      uiForm = (UITicketForm) event.getSource();
      String users = uiForm.getUIStringInput("users").getValue();
      if(users == "") users = null;
      String permission=uiForm.getUIStringInput("permission").getValue();
      if(permission == "") permission = null;
      String creator=uiForm.getUIStringInput("creator").getValue();
      if(creator == "") creator = null;
      String description = uiForm.getUIStringInput("description").getValue();
      String mail = uiForm.getUIStringInput("sendmail").getValue();
      if(description == "") description = null;      
      long accesstime =  uiForm.getUIDateTimeInput("accesstime").getCalendar().getTimeInMillis() 
      - new Date().getTime();            
      int accesslimit = -1;
      try{
        accesslimit = Integer.parseInt(uiForm.getUIStringInput("limit").getValue());
      }catch (NumberFormatException e) {
        throw new Exception("Input only number");
      }            
      Property prop = uiForm.node_.getProperty("jcr:primaryType") ;
      String nodeType = prop.getValue().toString() ;
      Information iprovider = findInformationProvider(uiForm) ;
      if(uiForm.node_.getPath().equals("/")) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
            res.getString("UITicketForm.msg.unable-create"))) ;
        return ;
      }      
      String nodePath = uiForm.node_.getPath().substring(1) ;      
      TicketCommand tc = new TicketCommandImpl();
      StringBuffer properties = new StringBuffer();
      properties.append("nodeType=").append(nodeType).append("\n")
                .append("nodePath=").append(nodePath).append("\n")
                .append("workspace=").append(uiForm.session_.getWorkspace().getName()).append("\n")
                .append("portalName=").append(portalName_) ;
      tc.setProperty(TicketCommand.PROPERTIES_NAME, properties.toString()) ;      
      if(!uiForm.create && accesstime>0){        
        String ticketId = uiForm.service_.createTicket(accesstime, accesslimit, description, creator,null,null,tc);
        uiForm.ticketId=ticketId;
        uiForm.setFieldValue("idticket",getUrl(ticketId,portalName_));
        LockForm();
      }
      
    }
    private void LockForm(){
      uiForm.limit_.setEditable(false);
      uiForm.creator_.setEditable(false);
      uiForm.desc_.setEditable(false);
      uiForm.permission_.setEditable(false);
      uiForm.user_.setEditable(false);
      uiForm.mail_.setEditable(false);
      uiForm.toDate_.setEditable(false);
      uiForm.create=true;
      uiForm.update=false;
    }
    
    public String getUrl(String ticketId,String portalId) throws Exception {      
      ExoPortletRequest request = 
        (ExoPortletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();                     
      String hostName =request.getServerName() ;
      String port = Integer.toString(request.getServerPort());
      String protocol = request.getScheme() ;
      String contextPath = request.getContextPath() ;
      StringBuffer url = new StringBuffer() ;      
      url.append(protocol).append("://").append(hostName).append(":").append(port)
            .append(contextPath).append("/ticket?ticketId=").append(ticketId)
            .append("&portalId=").append(portalId) ;      
      return url.toString();
    }
    
  }  
  
  static public class UpdateActionListerner extends ExoActionListener{
    private UITicketForm uiForm;
    String ticketID="";
    public void execute(ExoActionEvent event) throws Exception {
      uiForm = (UITicketForm) event.getSource();
      ticketID=uiForm.ticketId;
      if(!uiForm.update){
        UnLockForm();
      }      
    }
    private void UnLockForm(){
      uiForm.limit_.setEditable(true);
      uiForm.creator_.setEditable(true);
      uiForm.desc_.setEditable(true);
      uiForm.permission_.setEditable(true);
      uiForm.user_.setEditable(true);
      uiForm.mail_.setEditable(true);
      uiForm.toDate_.setEditable(true);
      uiForm.id_.setText("");
      uiForm.create=false;
      uiForm.update=true;
    }
    
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {      
      UITicketForm uicomp = (UITicketForm) event.getSource();
      uicomp.reset();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode() ;
      uicomp.setRendered(false) ;
      uiExplorer.broadcastOnChange(node);      
    }
  }
  
}
