package org.exoplatform.portlets.content.metadata.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;

import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.metadata.MetadataService;
import org.exoplatform.services.jcr.RepositoryService;
/**
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04
 */
public class UIMetadataManager extends UIExoCommand {
  
  final static public String METADATA_ACTION = "metadata";
  public static Parameter[] VIEW_PARAMS = { new Parameter(ACTION , VIEW_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  public static Parameter[] METADATA_PARAMS = { new Parameter(ACTION , METADATA_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
   
  private NodeTypeManager ntManager_ ;
  private CmsConfigurationService cmsConfigService_ ;
  private RepositoryService repositoryService_ ;
  private List metadatasName_ = new ArrayList() ;
  private MetadataService mservice_ ;
  private UIPageListIterator uiPageIterator_ ;  
  
  public UIMetadataManager(RepositoryService repositoryService, 
        CmsConfigurationService cmsConfigService, MetadataService mservice) throws Exception{
    setId("UIMetadataManager") ;
    setRendererType("VelocityRenderer") ;
    cmsConfigService_ = cmsConfigService ;
    repositoryService_ = repositoryService ; 
    mservice_ = mservice ;
    Session session = repositoryService_.getRepository().getSystemSession(cmsConfigService_.getWorkspace()) ;    
    ntManager_ = session.getWorkspace().getNodeTypeManager();
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(ViewActionListener.class, VIEW_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(MetadataActionListener.class, METADATA_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    uiPageIterator_.setPageList(new ContentPageList(getAllMetadatas()));  
  }  
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
  public Parameter[] getMetadataParams() { return METADATA_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  
  public List getAllMetadatas() {
     List<NodeType> metadatas = new ArrayList<NodeType>() ;     
    try{
      metadatasName_ = mservice_.getMetadataList() ;
      for(int i = 0; i < metadatasName_.size(); i ++) {
        metadatas.add(ntManager_.getNodeType(metadatasName_.get(i).toString())) ;
      }
    }catch(Exception e) {
      e.printStackTrace() ;
    }     
    return metadatas ; 
  }
  
  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getAllMetadatas()));        
  }
  
  public String getIcon(String nodeType) {
    return CMSUtils.getIconByNodeTypeName(nodeType) ;
  }
  
  static public class MetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager uicomp = (UIMetadataManager)event.getComponent() ;
      UIMappingMetadataForm metadataForm = (UIMappingMetadataForm) uicomp.getSibling(UIMappingMetadataForm.class) ;
      try {
        metadataForm.update(uicomp.ntManager_) ;
        uicomp.setRenderedSibling(UIMappingMetadataForm.class) ;
      }catch(Exception e) {
        e.printStackTrace();        
      }
            
    }
  }
  
  static public class ViewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager uicomp = (UIMetadataManager)event.getComponent() ;
      String metadataName = event.getParameter(OBJECTID) ;
      NodeType nodeType = uicomp.ntManager_.getNodeType(metadataName) ;
      UIMetadataView metadataView = (UIMetadataView)uicomp.getSibling(UIMetadataView.class) ;
      metadataView.setMetadata(nodeType) ;
      uicomp.setRenderedSibling(UIMetadataView.class) ;
    }
  }
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager uicomp = (UIMetadataManager)event.getComponent() ;
      String metadataName = event.getParameter(OBJECTID) ;
      UIMappingMetadataForm metadataForm = 
        (UIMappingMetadataForm)uicomp.getSibling(UIMappingMetadataForm.class) ;
      metadataForm.update(metadataName) ;
      uicomp.setRenderedSibling(UIMappingMetadataForm.class) ;
    }
  }
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager uicomp = (UIMetadataManager)event.getComponent() ;
      String metadataName = event.getParameter(OBJECTID) ;
      uicomp.mservice_.removeMetadata(metadataName) ;
      uicomp.refresh() ;
    }
  }
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataManager metadadaManager = (UIMetadataManager)event.getComponent() ;
      metadadaManager.setRenderedSibling(UIECMAdminSummary.class) ;
    }
  }
}
