package org.exoplatform.portlets.content.queries.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.Value;
import javax.jcr.query.Query;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.PermissionValidator;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.queries.QueryService;

/**
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIMappingMetadataForm.java,v 1.2 2006/04/04
 */
public class UIQueriesForm extends UISimpleForm implements SelectorHandler{
  
  final public static String SQL_QUERY = "select * from exo:article where jcr:path like '/cms/publications/%'" ;
  final public static String XPATH_QUERY = "/jcr:root/cms/publications//element(*, exo:article)" ;
  final public static String STATEMENT = "statement" ;
  final public static String LANGUAGE = "language" ;
  final public static String QUERY_NAME = "queryName" ;
  final public static String CACHED_RESULT = "cachedResult" ;
  final public static String VIEW_PERMISSIONS = "viewPermission" ;
  final public static String CHANGE_QUERY_ACTION = "changeQuery" ;
  static String  SEARCH_GROUP = "searchGroup" ;
  private UIExoPermissionSelector permSelector_;
  private QueryService qservice_ ;
  
  public UIQueriesForm(QueryService qservice) throws Exception{    
    super("UIQueriesForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");    
    qservice_ = qservice ;
    UIStringInput queryName = new UIStringInput(QUERY_NAME, "") ;
    add(queryName) ;
    
    UICheckBox cacheResult = new UICheckBox(CACHED_RESULT, "") ;
    add(cacheResult) ;
    
    UITextArea statement = new UITextArea(STATEMENT, "") ;
    statement.setValue(XPATH_QUERY) ;
    add(statement) ;
    
    List<SelectItem> options = new ArrayList<SelectItem>();    
    options.add(new SelectItem(Query.SQL, Query.SQL));
    options.add(new SelectItem(Query.XPATH, Query.XPATH));
    UISelectBox queryLanguage = new UISelectBox(LANGUAGE,Query.XPATH, options);
    queryLanguage.setUpdateOnChangeAction(CHANGE_QUERY_ACTION) ;
    add(queryLanguage);
    
    permSelector_ = new UIExoPermissionSelector(VIEW_PERMISSIONS, null) ;
    //permSelector_.addValidator(PermissionValidator.class) ;
    add(permSelector_) ;
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION); 
    addActionListener(ChangeQueryActionListener.class, CHANGE_QUERY_ACTION);    
  }
  
  public String getSearchGroup() { return SEARCH_GROUP; }
  
  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIQueriesForm.class) ;
  }
  
  public void select(Selector selector) throws Exception {
    UIExoPermissionBrowser browser = (UIExoPermissionBrowser)selector ;
    if(browser.isGroup()) {
      permSelector_.setPermission(selector.getInputValue());
    }else {
      permSelector_.setValue(browser.getSelectedUser()) ;
    }
    selector.setRenderedSibling(UIQueriesForm.class);    
  }
  
  public void update(String queryName)throws Exception{
    if(queryName != null) {
      Node query = qservice_.getSharedQuery(queryName) ;
      getUIStringInput(QUERY_NAME).setValue(queryName).setEditable(false) ;
      getUICheckBox(CACHED_RESULT).setSelect(query.getProperty("exo:cachedResult").getBoolean()) ;
      getUIInput(STATEMENT).setValue(query.getProperty("jcr:statement").getString()) ;
      Value[] values = query.getProperty("exo:permissions").getValues() ;
      String strValues = "" ;
      for(int i = 0; i < values.length; i ++) {
        if(i > 0) strValues = strValues + ", " + values[i].getString() ;
        else strValues = strValues + values[i].getString() ;
      }
      permSelector_.setValue(strValues) ;      
    }else {
      getUIStringInput(QUERY_NAME).setValue("").setEditable(true) ;
      getUIInput(LANGUAGE).setValue(Query.XPATH) ;
      getUICheckBox(CACHED_RESULT).setSelect(false) ;
      getUIInput(STATEMENT).setValue(XPATH_QUERY) ;
      permSelector_.setValue("") ;
    }    
  }
  
  public static class ChangeQueryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueriesForm uiForm= (UIQueriesForm)event.getSource() ;
      String language = uiForm.getUIInput(LANGUAGE).getValue() ;
      if(language.equals(Query.XPATH)) {
        uiForm.getUIInput(STATEMENT).setValue(uiForm.XPATH_QUERY) ;
      }else {
        uiForm.getUIInput(STATEMENT).setValue(uiForm.SQL_QUERY) ;
      }
    }    
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try{
        UIQueriesForm uiForm = (UIQueriesForm) event.getSource();      
        boolean cachedResult = uiForm.getUICheckBox(CACHED_RESULT).getSelect() ; 
        String queryName = uiForm.getUIStringInput(QUERY_NAME).getValue() ;
        String statement = uiForm.getUIInput(STATEMENT).getValue() ;
        String permissions = uiForm.permSelector_.getValue() ;
        String language = uiForm.getUIInput(LANGUAGE).getValue() ;
        if(queryName == null || queryName.length() < 1 || statement == null || statement.length() < 1
            || permissions == null || permissions.length() < 1) {
          ResourceBundle res = getApplicationResourceBundle() ;
          Information iprovider = findInformationProvider(uiForm);
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
              res.getString("UIQueriesForm.msg.all-fields-required"))) ;
          return ;
        }
        if(permissions.indexOf(",") > -1) {
          uiForm.qservice_.addSharedQuery(queryName, statement, language, permissions.split(","), cachedResult) ;  
        }else {
          uiForm.qservice_.addSharedQuery(queryName, statement, language, new String[] {permissions}, cachedResult) ;
        }              
        UIQueriesManager queriesManager = (UIQueriesManager)uiForm.getSibling(UIQueriesManager.class) ;
        queriesManager.refresh() ;
        uiForm.setRenderedSibling(UIQueriesManager.class) ;
      }catch(Exception e) {
        e.printStackTrace() ;
      }      
    }  
  }
  
  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueriesForm uiForm = (UIQueriesForm) event.getSource();
      UIExoPermissionBrowser uiGroupBrowser = 
        (UIExoPermissionBrowser) uiForm.getSibling(UIExoPermissionBrowser.class);           
      uiGroupBrowser.setHandler(uiForm);
      uiForm.setRenderedSibling(UIExoPermissionBrowser.class);  
    }
  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueriesForm uiForm = (UIQueriesForm) event.getSource();
      uiForm.setRenderedSibling(UIQueriesManager.class) ;
    }
  }
}
