package org.exoplatform.portlets.content.queries.component;

import java.util.Collection;
import java.util.List;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.cms.queries.QueryService;
/**
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04
 */
public class UIQueriesManager extends UIExoCommand {
  
  final static public String ADD_ACTION = "add";
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
   
  private UIPageListIterator uiPageIterator_ ;  
  private QueryService qservice_ ;
  
  public UIQueriesManager(QueryService qservice) throws Exception{
    setId("UIQueriesManager") ;
    setRendererType("VelocityRenderer") ;
    qservice_ = qservice ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    uiPageIterator_.setPageList(new ContentPageList(getAllQueries()));  
  }  
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  
  public List getAllQueries() throws Exception{
    return qservice_.getSharedQueries() ; 
  }
  
  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getAllQueries()));        
  }
  
  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueriesManager uicomp = (UIQueriesManager)event.getComponent() ;
      UIQueriesForm queriesForm = (UIQueriesForm) uicomp.getSibling(UIQueriesForm.class) ;
      queriesForm.update(null) ;
      uicomp.setRenderedSibling(UIQueriesForm.class) ;            
    }
  }
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try{
        UIQueriesManager uicomp = (UIQueriesManager)event.getComponent() ;
        String queriesName = event.getParameter(OBJECTID) ;
        UIQueriesForm queriesForm = (UIQueriesForm) uicomp.getSibling(UIQueriesForm.class) ;
        queriesForm.update(queriesName) ;
        uicomp.setRenderedSibling(UIQueriesForm.class) ;
      }catch(Exception e) {
        e.printStackTrace() ;
      }
    }
  }
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueriesManager uicomp = (UIQueriesManager)event.getComponent() ;
      String queriesName = event.getParameter(OBJECTID) ;
      uicomp.qservice_.removeQuery(queriesName) ;
      uicomp.refresh() ;
    }
  }
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueriesManager queriesManager = (UIQueriesManager)event.getComponent() ;
      queriesManager.setRenderedSibling(UIECMAdminSummary.class) ;
    }
  }
}
