package org.exoplatform.portlets.content.script.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.version.VersionException;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.versions.component.VersionNode;
import org.exoplatform.services.cms.scripts.ScriptService;

public class UIScriptForm extends UISimpleMulltipartForm {
  
  static String SCRIPT_NAME = "script".intern(), CONTENT = "content".intern() , 
                SCRIPT_TYPE = "type".intern() ;  
  static String SCRIPT_ENABLE_VERSIONING = "enableVersioning";
  static String RESTORE_VERSION_ACTION = "restoreVersion".intern() ;
  
  protected ScriptService scriptService_;  
  protected UIStringInput scriptNameInput_;
  protected UISelectBox selectedTypeBox_ ;
  protected UITextArea scriptContentInput_;
  protected boolean isECMScript_ ;    
  protected UICheckBox scriptEnableVersioning_;
  protected boolean newScript_;
  protected Node node_;
  protected VersionNode selectedVersion_;
  private Node frozenNode_ ;
  
  public UIScriptForm(ScriptService scriptService) {
    super("scriptForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    
    scriptService_ = scriptService;    
    scriptNameInput_ = new UIStringInput(SCRIPT_NAME, ""); 
    scriptContentInput_ = new UITextArea(CONTENT, "");
    scriptContentInput_.setClazz("large");
    selectedTypeBox_ = new UISelectBox(SCRIPT_TYPE,"",new ArrayList<SelectItem>()) ;
    
    add(scriptNameInput_) ;
    add(scriptContentInput_) ;    
    add(selectedTypeBox_) ;
    
    scriptEnableVersioning_ = new UICheckBox(SCRIPT_ENABLE_VERSIONING, null);    
    scriptEnableVersioning_.setSelect(false) ;
    add(scriptEnableVersioning_) ;    
    
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);    
    addActionListener(RestoreVersionActionListener.class, RESTORE_VERSION_ACTION); 
  }
  
  public String getRestoreVersionAction() { return RESTORE_VERSION_ACTION; }
  
  public boolean isNewScript() throws Exception {
    return newScript_;
  }
  
  public boolean isVersioned() throws RepositoryException {    
    if (!newScript_) {     
      return node_.isNodeType("mix:versionable");
    }    
    return false;
  }
  
  public boolean isEnableRestoring() throws Exception {    
    if (isVersioned()) {
      long versionSize = node_.getVersionHistory().getAllVersions().getSize();
      if (versionSize > 2) return true;     
    }
    return false;
  }
  
  public boolean canEnableVersionning() throws Exception {
    return node_.canAddMixin("mix:versionable");
  }
  
  public void setData(boolean isECMScript, String scriptName,
      List<SelectItem> options, String selectedScriptTypeValue, VersionNode selectedVersion) throws Exception{    
    isECMScript_ = isECMScript ;                
    selectedTypeBox_.setOptions(options) ;    
    getUISelectBox(SCRIPT_TYPE).setValue(selectedScriptTypeValue) ;   
    if(scriptName == null || scriptName.length() == 0) {
      scriptNameInput_.setText("").setEditable(true);
      scriptContentInput_.setText("");
      selectedTypeBox_.setReadonly(false) ;
      newScript_ = true;
    }else {
      String name2get = selectedScriptTypeValue + "/" + scriptName ;      
      String script =  scriptService_.getScriptAsText(name2get);
      scriptNameInput_.setText(scriptName).setEditable(false);
      scriptContentInput_.setText(script);
      selectedTypeBox_.setReadonly(true) ;
      node_ = scriptService_.getScriptNode(name2get);  
      newScript_ = false;
      if (isVersioned()) {     
        if (selectedVersion != null) {
          selectedVersion_ = selectedVersion;
          frozenNode_ = selectedVersion_.getVersion().getNode("jcr:frozenNode") ;          
          script = frozenNode_.getProperty("jcr:data").getString();                   
          scriptNameInput_.setText(scriptName).setEditable(false);          
          scriptContentInput_.setText(script);                                      
        }        
        scriptEnableVersioning_.setSelect(true);
        scriptEnableVersioning_.setEditable(false);
      }else if (canEnableVersionning()){       
        scriptEnableVersioning_.setSelect(false);
        scriptEnableVersioning_.setEditable(true);
      }
    }   
  }
  
  static public class RestoreVersionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptForm uiForm = (UIScriptForm) event.getSource();        
      ResourceBundle res = getApplicationResourceBundle() ;      
      Information iprovider = findInformationProvider(uiForm);
      try {                      
        if (uiForm.selectedVersion_ != null) {
          uiForm.node_.restore(uiForm.selectedVersion_.getVersion(), true);
          uiForm.node_.checkout();
          Object[] args = {uiForm.selectedVersion_.getName()};        
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
                         res.getString("UIScriptForm.msg.version-restored"), args));
        }
        if(uiForm.isECMScript_) {       
          UIScriptFormController scriptFormController = (UIScriptFormController) uiForm.getParent();            
          UIECMScriptList ecmScriptList = (UIECMScriptList) scriptFormController.getSibling(UIECMScriptList.class);                                
          ecmScriptList.refresh();
          ecmScriptList.setRenderedSibling(UIECMScriptList.class);                 
        }else {
          UIScriptFormController scriptFormController = (UIScriptFormController) uiForm.getParent();            
          UICBScriptList cbScriptList = (UICBScriptList) scriptFormController.getSibling(UICBScriptList.class);                                
          cbScriptList.refresh();
          cbScriptList.setRenderedSibling(UICBScriptList.class);                    
        }  
      } catch (VersionException ex) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,ex.getMessage()));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIScriptForm uiForm = (UIScriptForm) event.getSource();      
      String scriptName = uiForm.scriptNameInput_.getText();
      String text = uiForm.scriptContentInput_.getText();
      String preScriptName = uiForm.selectedTypeBox_.getValue() ;
      Information iprovider = findInformationProvider(uiForm);
      if (scriptName == null || scriptName.length() < 1) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
            res.getString("UIScriptForm.msg.view-name-invalid"))) ;
        return ;
      }
      try {        
        String name2save = preScriptName + "/" +scriptName ;        
        uiForm.scriptService_.addScript(name2save,text) ;        
        Node node = uiForm.scriptService_.getScriptNode(name2save);        
        if (!uiForm.newScript_) {                   
          boolean isScriptEnableVersioning = uiForm.scriptEnableVersioning_.getSelect();          
          if (isScriptEnableVersioning) {            
            if (uiForm.canEnableVersionning() && !uiForm.isVersioned()) {              
              node.addMixin("mix:versionable");              
              node.save();                                  
            }                                  
            node.checkin();
            node.checkout();                
            node.save();
          }
        }                  
        if(uiForm.isECMScript_) {       
          UIScriptFormController scriptFormController = (UIScriptFormController) uiForm.getParent();            
          UIECMScriptList ecmScriptList = (UIECMScriptList) scriptFormController.getSibling(UIECMScriptList.class);                                
          ecmScriptList.refresh();
          ecmScriptList.setRenderedSibling(UIECMScriptList.class);                 
        }else {
          UIScriptFormController scriptFormController = (UIScriptFormController) uiForm.getParent();            
          UICBScriptList cbScriptList = (UICBScriptList) scriptFormController.getSibling(UICBScriptList.class);                                
          cbScriptList.refresh();
          cbScriptList.setRenderedSibling(UICBScriptList.class);                    
        }
      }catch (Exception e) {        
        JCRExceptionManager.process(iprovider,e) ;
      }                              
    }
  }    
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIScriptForm uiForm = (UIScriptForm) event.getSource();
      UIScriptFormController scriptFormController = (UIScriptFormController) uiForm.getParent();
      if(uiForm.isECMScript_) {
        UIECMScriptList ecmScriptList = (UIECMScriptList) scriptFormController.getSibling(UIECMScriptList.class);         
        ecmScriptList.setRenderedSibling(UIECMScriptList.class);              
      }else {
        UICBScriptList cbScriptList = (UICBScriptList) scriptFormController.getSibling(UICBScriptList.class);         
        cbScriptList.setRenderedSibling(UICBScriptList.class);           
      }             
    }
  }  
  
}
