/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.script.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.version.VersionHistory;

import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.versions.component.VersionNode;
import org.exoplatform.services.cms.scripts.ScriptService;

/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 * @email:   jobdung@yahoo.com
 * @version: $Id: UISelectScriptVersionForm.java,v 1.0 2006/08/23 18:11:26 dunghoang Exp $
 */

public class UISelectScriptVersionForm extends UISimpleForm  {
 
  final static public String CHANGE_VERSION = "changeVersion" ;  
  protected UISelectBox uiSelect_ ;  
  private String defaultNodeVersion_;
  private ScriptService scriptService_;    
  protected VersionNode rootVersion_;  
  protected Node node_;    
  protected List<VersionNode> listNodeVersion_ = new ArrayList<VersionNode>() ;
  protected boolean isECMScript_;
  protected String scriptName_;
  protected List<SelectItem> options_; 
  protected String selectedScriptTypeValue_;
  
  public UISelectScriptVersionForm(ScriptService scriptService) throws Exception {
    super("selectScriptVersionForm", "post", null) ;    
    setRendererType("SimpleFormVelocityRenderer");        
    uiSelect_ = new  UISelectBox("version", "", null) ;
    uiSelect_.setUpdateOnChangeAction(CHANGE_VERSION) ;
    add(uiSelect_) ;    
    scriptService_ = scriptService;    
    addActionListener(ChangeTypeActionListener.class, CHANGE_VERSION) ;    
  }  
 
  public void setScript(boolean isECMScript, String scriptName,
      List<SelectItem> options, String selectedScriptTypeValue) throws Exception{    
    String name2get = selectedScriptTypeValue + "/" + scriptName ;    
    node_ = scriptService_.getScriptNode(name2get);  
    isECMScript_ = isECMScript;
    scriptName_ = scriptName;
    options_ = options;        
    selectedScriptTypeValue_ = selectedScriptTypeValue;
    rootVersion_ = null;   
    this.update();          
  }
  
  public VersionNode getRootVersion() throws Exception{        
    if (rootVersion_ == null) {
      VersionHistory vH;
      try {        
        vH = node_.getVersionHistory();
        rootVersion_ = new VersionNode(vH.getRootVersion());        
      } catch (RepositoryException e) {    
        return null;
      }      
    }            
    return rootVersion_;
  }
       
  private List<VersionNode> getNodeVersions(List<VersionNode> children) throws Exception {         
    List<VersionNode> child = new ArrayList<VersionNode>() ;
    for(int i = 0; i < children.size(); i ++){      
      listNodeVersion_.add(children.get(i));            
      child = children.get(i).getChildren();
      if (!child.isEmpty()) 
        getNodeVersions(child); 
    }           
    return listNodeVersion_;   
  }
   
  public String getDefaultVersionNode() throws Exception{          
    defaultNodeVersion_ = node_.getBaseVersion().getName();
    return defaultNodeVersion_;
  }  
  
  public void update() throws Exception {       
    List<SelectItem> options = new ArrayList<SelectItem>();
    List<VersionNode> children = getRootVersion().getChildren();    
    listNodeVersion_.clear();    
    List<VersionNode> nodeVersions = getNodeVersions(children) ;
    List<VersionNode> sortedNodeVersions = new ArrayList<VersionNode>() ;
    
    for (int i = 0; i < nodeVersions.size(); i ++){            
      VersionNode minVersion = nodeVersions.get(i);
      VersionNode currentVersion = nodeVersions.get(i);
      for (int j = i+1; j < nodeVersions.size(); j ++){
       if (Integer.parseInt(nodeVersions.get(j).getName()) < Integer.parseInt(minVersion.getName())) {         
         minVersion = nodeVersions.get(j);         
       }       
      }      
      sortedNodeVersions.add(minVersion);            
      int indexCur = nodeVersions.indexOf(currentVersion);
      int indexMin = nodeVersions.indexOf(minVersion);      
      nodeVersions.add(indexCur,minVersion);
      nodeVersions.remove(indexCur+1);            
      nodeVersions.add(indexMin,currentVersion);
      nodeVersions.remove(indexMin+1);      
    }              
    
    for(int i = 0; i < sortedNodeVersions.size(); i ++){
      options.add(new SelectItem(sortedNodeVersions.get(i).getName().toString(), sortedNodeVersions.get(i).getName().toString()));
    }
    
    uiSelect_.setOptions(options) ;            
    uiSelect_.setValue(getDefaultVersionNode());  
  }
  
  static public class ChangeTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectScriptVersionForm uicomp = (UISelectScriptVersionForm) event.getSource();      
      String path = uicomp.node_.getVersionHistory().getVersion(uicomp.uiSelect_.getValue()).getPath() ;           
      VersionNode versionNode = uicomp.rootVersion_.findVersionNode(path);             
      boolean isECMScript = uicomp.isECMScript_;
      String scriptName = uicomp.scriptName_;
      List<SelectItem> options = uicomp.options_;      
      String selectedScriptTypeValue = uicomp.selectedScriptTypeValue_;      
      UIScriptForm scriptForm = (UIScriptForm) uicomp.getSibling(UIScriptForm.class) ;        
      scriptForm.setData(isECMScript, scriptName, options, selectedScriptTypeValue, versionNode) ;      
    }
  }

}