/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.search.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.queries.QueryService;

/**
 * @author NguyenKeQuangHung (nguyenkequanghung@yahoo.com)
 * @since jan 18, 2007
 * @version $Id$
 */
public class UIAdvanceConstraints extends UISimpleForm  implements ChangeNodeListener{

  public static final String SELECT_METADATA_ACTION = "metadataActions" ;
  public static final String CONTAINS = "contains" ; 
  public static final String NOT_CONTAINS = "notcontains" ;
  public static final String METADATA_PROPERTY = "metadataProperty" ;
  public static final String OPERATOR = "operator" ;
  
  
  
  private UIStringInput metadata_ ;
  protected QueryService queryService_;
  protected String query_;

  
  public UIAdvanceConstraints(QueryService queryService) throws Exception {
    super("UIAdvanceContraints", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    queryService_ = queryService;    
    UIStringInput contains = new UIStringInput(CONTAINS, "");
    UIStringInput notContains = new UIStringInput(NOT_CONTAINS, "");
    UISelectBox operator = new UISelectBox(OPERATOR, "", null);
    List<SelectItem> options = new ArrayList<SelectItem> () ;
    options.add(new SelectItem("And", "and")) ;
    options.add(new SelectItem("Or", "or")) ;
    metadata_ = new UIStringInput(METADATA_PROPERTY, "");
    add(metadata_) ;
    add(contains);
    add(notContains) ;
    operator.setOptions(options) ;
    add(operator) ;
    
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(SelectMetadataActionListener.class, SELECT_METADATA_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public String selectMetadataParams() { return SELECT_METADATA_ACTION ;  }
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false) ;
  }
  
  public void updateMetadata(List selectedProperies) {
    String value = getUIStringInput(METADATA_PROPERTY).getValue() ;
    if(value == null) value = "" ;
    StringBuffer selectedValue = new StringBuffer() ;
    for(int i = 0 ; i < selectedProperies.size(); i ++) {
      if(value.indexOf(selectedProperies.get(i).toString()) == -1) {
        if (selectedValue.length() > 0) selectedValue.append(", ") ;
        selectedValue.append(selectedProperies.get(i)) ;
      }
    }
    if(value.length() > 0 ) {
      if(selectedValue.length() > 0) metadata_.setValue(value + ", " + selectedValue.toString()) ;
      else metadata_.setValue(value) ;
    }else {
      metadata_.setValue(selectedValue.toString()) ;
    } 
  }
  
  static public class SelectMetadataActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdvanceConstraints uiForm = (UIAdvanceConstraints) event.getComponent() ;
      UISelectMetadataForm metadataForm = 
        (UISelectMetadataForm) uiForm.getSibling(UISelectMetadataForm.class);
      String properties = uiForm.getUIStringInput(METADATA_PROPERTY).getValue() ;
      List<String> selected = new ArrayList<String> () ;
      if(properties != null && properties.length() > 0) {
        String[] array = properties.split(",") ;
        for(int i = 0; i < array.length; i ++) {
          selected.add(array[i].trim()) ;
        }
      }
      metadataForm.update(selected) ;
      uiForm.setRenderedSibling(UISelectMetadataForm.class) ;
    }
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdvanceConstraints uiForm = (UIAdvanceConstraints) event.getSource();
      String operator = uiForm.getUISelectBox(OPERATOR).getValue() ;
      String properties = uiForm.metadata_.getValue() ;
      if(properties == null || properties.length() < 1) {
        ResourceBundle res = getApplicationResourceBundle() ;
        Information iprovider = findInformationProvider(uiForm);
        iprovider.addMessage(
            new ExoFacesMessage(FacesMessage.SEVERITY_WARN,res.getString("UIAdvanceConstraints.msg.properties-required"))) ;
        return ;
      }
      String contains = uiForm.getUIStringInput(CONTAINS).getValue() ;
      if(contains == null) contains = "" ;
      else contains = contains.trim() ;
      String notContains = uiForm.getUIStringInput(NOT_CONTAINS).getValue() ;
      if (notContains == null ) notContains = "" ;
      else notContains = notContains.trim(); 
      String advanceQuery = "" ;
      if(properties.indexOf(",") > -1) {
        String[] array = properties.split(",") ;
        for(String property : array) {
          if(advanceQuery.length() > 0) advanceQuery = advanceQuery + operator ;
          advanceQuery = advanceQuery + getQuery(property, contains, notContains) ;
        }
      }else {
        advanceQuery = getQuery(properties, contains, notContains) ;
      }
      UISearch uiSearch = (UISearch)uiForm.getSibling(UISearch.class) ;
      UISimpleSearch simpleSearch = 
        (UISimpleSearch)uiSearch.getChildComponentOfType(UISimpleSearch.class) ;
      simpleSearch.updateAdvanceConstraint(advanceQuery) ;
      uiForm.metadata_.setValue("") ;
      uiForm.getUIStringInput(CONTAINS).setValue("") ;
      uiForm.getUIStringInput(NOT_CONTAINS).setValue("") ;      
      uiForm.setRenderedSibling(UISearch.class) ;
    }
    
    private String getQuery(String property, String contains, String notContains) {
      String advanceQuery = "" ;
      if(contains.length() > 0 && notContains.length() > 0) {
        advanceQuery =" contains(" + property.trim() + ", '"+ contains + " -" + notContains + "') " ;
      }else if(contains.length() > 0){
        advanceQuery =" contains(" + property.trim() + ", '"+ contains + "') " ;
      }else if(notContains.length() > 0){
        advanceQuery = " not(contains(" + property.trim() + ", '"+ notContains + "')) " ;
      }
      return advanceQuery ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAdvanceConstraints uiForm = (UIAdvanceConstraints) event.getComponent() ;
      uiForm.setRenderedSibling(UISearch.class) ;      
    }
  }  
}
