package org.exoplatform.portlets.content.search.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.QueryResult;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.queries.QueryService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.access.AccessControlEntry;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;

public class UIQueryList extends UIExoCommand {

  public static final String EXECUTE_ACTION = "execute".intern();

  public static Parameter[] EXECUTE_PARAMS = { new Parameter(ACTION,
      EXECUTE_ACTION) };
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION,
      DELETE_ACTION) };
  private List roles_ = new ArrayList() ;
  protected QueryService queryService_;
  private RepositoryService repositoryService_ ;
  private CmsConfigurationService cmsConf_ ;
  private List<Node> sharedQueries_ = new ArrayList<Node> ();
  public UIQueryList(QueryService queryService, OrganizationService oservice, 
      RepositoryService repositoryService, CmsConfigurationService cmsConf) throws Exception{
    setId("UIQueryList");
    setRendererType("VelocityRenderer");
    repositoryService_ = repositoryService ;
    cmsConf_ = cmsConf ;
    queryService_ = queryService;
    FacesContext context  = FacesContext.getCurrentInstance() ;
    ExternalContext econtext = context.getExternalContext()  ;
    String userName = econtext.getRemoteUser() ;
    Collection memberships = oservice.getMembershipHandler().findMembershipsByUser(userName) ;
    if(memberships != null && memberships.size() > 0){
      Object[] objects = memberships.toArray() ;      
      for(int i = 0 ; i < objects.length ; i ++ ){
        Membership membership = (Membership)objects[i] ;
        String role = membership.getMembershipType() + ":" + membership.getGroupId() ;
        roles_.add(role) ;
      } 
    }
    roles_.add(userName) ;
    addActionListener(ExecuteActionListener.class, EXECUTE_ACTION);
    addActionListener(DeleteActionListener.class, DELETE_ACTION);
  }

  public Parameter[] getExecuteParams() {
    return EXECUTE_PARAMS;
  }

  public Parameter[] getDeleteParams() {
    return DELETE_PARAMS;
  }

  public boolean hasQueries() throws Exception {
    List<Query> queries = queryService_.getQueries();
    if (queries == null || queries.isEmpty())
      return false;
    return true;
  }

  public List<Query> getQueries() throws Exception {
    return queryService_.getQueries();
  }
  
  public boolean hasSharedQueries() throws Exception {
    if(roles_.size() > 0) {
      sharedQueries_ = queryService_.getSharedQueriesByPermissions(roles_) ;
      if(sharedQueries_.size() > 0) return true ;      
    }
    return false ;            
  }
  
  public List<Node> getSharedQueries() { return sharedQueries_ ; }
  
  public List<PermissionBean> getPermissions(String path) throws Exception {
    List<PermissionBean> permBeans = new ArrayList<PermissionBean>();
    Session session = repositoryService_.getRepository().getSystemSession(cmsConf_.getWorkspace());
    Node node = (Node)session.getItem(path) ;      
    List globalPermissionsList = ((ExtendedNode)node).getACL().getPermissionEntries();
    Map identitiesAndPermissionsMap = new HashMap();
    for (Iterator globalPermissionsListIterator = globalPermissionsList.iterator(); globalPermissionsListIterator.hasNext();) {
      AccessControlEntry accessControlEntry = (AccessControlEntry)globalPermissionsListIterator.next();
      String currentIdentity = accessControlEntry.getIdentity();
      String currentPermission = accessControlEntry.getPermission();
      List currentPermissionsList = (List)identitiesAndPermissionsMap.get(currentIdentity);
      if (identitiesAndPermissionsMap.containsKey(currentIdentity) == false) {
        identitiesAndPermissionsMap.put(currentIdentity, null);
      }
      if (currentPermissionsList == null){
        currentPermissionsList = new ArrayList();
      }  
      if (currentPermissionsList.contains(currentPermission) == false) {
        currentPermissionsList.add(currentPermission);
      }
      identitiesAndPermissionsMap.put(currentIdentity,currentPermissionsList);
    }
    //at this stage we have Map <name, (List)permissions>
    Set keys = identitiesAndPermissionsMap.keySet();  
    for (Iterator tmpIterator = keys.iterator(); tmpIterator.hasNext();) {
      String userOrGroup = (String) tmpIterator.next();
      List permissions = (List) identitiesAndPermissionsMap.get(userOrGroup);
      PermissionBean permBean = new PermissionBean();
      permBean.setUsersOrGroups(userOrGroup);
      for (Iterator iterator = permissions.iterator(); iterator.hasNext();) {
        String perm = (String)iterator.next(); 
        if(PermissionType.READ.equals(perm))
          permBean.setRead(true);
        else if(PermissionType.ADD_NODE.equals(perm))
          permBean.setAddNode(true);
        else if(PermissionType.SET_PROPERTY.equals(perm))
          permBean.setSetProperty(true);
        else if(PermissionType.REMOVE.equals(perm))
          permBean.setRemove(true);
      }
      permBeans.add(permBean);
    }      
    return permBeans;
  }
  
  static public class ExecuteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryList uicomp = (UIQueryList) event.getComponent();
      String queryPath = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        //Query query = uicomp.queryService_.getQuery(queryPath);
        long beforeTime = System.currentTimeMillis();
        QueryResult queryResult = uicomp.queryService_.execute(queryPath);
        long duration = System.currentTimeMillis() - beforeTime;
        UIResults results = (UIResults) uicomp.getSibling(UIResults.class);
        results.resultMap_.clear() ;
        results.setQueryResults(duration, queryResult);
        uicomp.setRenderedSibling(UIResults.class);
      } catch (Exception e) {
        e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIQueryList uicomp = (UIQueryList) event.getComponent();
      String queryPath = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.queryService_.removeQuery(queryPath);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  public class PermissionBean {    
    private String usersOrGroups;
    private boolean read;
    private boolean addNode;
    private boolean setProperty;
    private boolean remove;    
    
    public String getUsersOrGroups() {
      return usersOrGroups;
    }
    public void setUsersOrGroups(String usersOrGroups) {
      this.usersOrGroups = usersOrGroups;
    }
    
    public boolean isAddNode() {
      return addNode;
    }
    public void setAddNode(boolean addNode) {
      this.addNode = addNode;
    }
    public boolean isRead() {
      return read;
    }
    public void setRead(boolean read) {
      this.read = read;
    }
    public boolean isRemove() {
      return remove;
    }
    public void setRemove(boolean remove) {
      this.remove = remove;
    }
    public boolean isSetProperty() {
      return setProperty;
    }
    public void setSetProperty(boolean setProperty) {
      this.setProperty = setProperty;
    }
  }
}
