/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.search.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.RepositoryService;

/**
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04 
 */
public class UISelectMetadataForm extends UISimpleMulltipartForm  implements ChangeNodeListener{
  
  final static public String CHANGE_METADATA_ACTION = "changeMetadata" ;
  final static public String REFRESH_ACTION = "refresh" ;
  private Session session_ ;
  private String  metadata_ = "dc:elementSet" ;
  private CmsConfigurationService cmsConfigurationService_ ;
  private TemplateService tservice_ ;
  private UISelectBox uiSelect_ ;
  private List<String> propertyList_ = new ArrayList<String>() ;
  private List selected_ = new ArrayList() ;
  private List<SelectItem> options_ ;
  public UISelectMetadataForm(RepositoryService repositoryService, TemplateService tservice,
                        CmsConfigurationService cmsConfigurationService) throws Exception {
    
    super("uiMetadataForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    cmsConfigurationService_ = cmsConfigurationService ;
    tservice_ = tservice ;
    session_ = repositoryService.getRepository().getSystemSession(cmsConfigurationService_.getWorkspace()) ;
    uiSelect_ = new  UISelectBox("metadataType", "", null) ;
    uiSelect_.setUpdateOnChangeAction(CHANGE_METADATA_ACTION) ;    
    options_ = new ArrayList<SelectItem>();
    String metadataPath = cmsConfigurationService_.getJcrPath(BasePath.METADATA_PATH);
    Node metadataHome = (Node)session_.getItem(metadataPath) ;
    NodeIterator iter = metadataHome.getNodes() ;
    while(iter.hasNext()) {
      Node meta = iter.nextNode() ;
      options_.add(new SelectItem(meta.getName(), meta.getName())) ;
    }
    List documentTemplates = tservice.getDocumentTemplates() ;
    for(int i = 0; i < documentTemplates.size(); i ++) {
      String nodeTypeName = documentTemplates.get(i).toString() ;
      options_.add(new SelectItem(nodeTypeName, nodeTypeName)) ;
    }
    addActionListener(ChangeMetadataActionListener.class, CHANGE_METADATA_ACTION) ;
    addActionListener(RefreshActionListener.class, REFRESH_ACTION) ;
    addActionListener(SaveMetadataActionListener.class, SAVE_ACTION);
    addActionListener(CancelMetadataActionListener.class, CANCEL_ACTION);
  }
  
  public String getRefreshParameter() { return REFRESH_ACTION ;}
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false) ;
  }
  
  public void update(List selected) throws Exception {
    getChildren().clear() ;
    uiSelect_.setOptions(options_) ;
    uiSelect_.setValue(metadata_);
    add(uiSelect_) ;
    UICheckBox checkbox ;
    NodeTypeManager ntManager = session_.getWorkspace().getNodeTypeManager() ;
    NodeType nt = ntManager.getNodeType(metadata_) ;
    propertyList_.clear() ;
    PropertyDefinition[] properties = nt.getPropertyDefinitions() ;
    for(int i = 0 ; i < properties.length ; i ++) {
      String name = properties[i].getName() ;
      propertyList_.add(name) ;
      checkbox = new UICheckBox(name, "") ;
      if(selected.contains(name)) checkbox.setSelect(true) ;
      add(checkbox) ;
    }
    selected_ = selected ;
  }
  
  public List getProperties() throws Exception {
    return propertyList_ ;
  }
  
  static public class ChangeMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectMetadataForm uicomp = (UISelectMetadataForm) event.getSource() ;
      uicomp.metadata_ = uicomp.uiSelect_.getValue() ;
      uicomp.update(uicomp.selected_) ;
    }
  }
  
  static public class RefreshActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectMetadataForm uicomp = (UISelectMetadataForm) event.getSource() ;
      String metadataPath = uicomp.cmsConfigurationService_.getJcrPath(BasePath.METADATA_PATH);
      Node metadataHome = (Node)uicomp.session_.getItem(metadataPath) ;
      NodeIterator iter = metadataHome.getNodes() ;
      while(iter.hasNext()) {
        Node meta = iter.nextNode() ;
        uicomp.options_.add(new SelectItem(meta.getName(), meta.getName())) ;
      }
      List documentTemplates = uicomp.tservice_.getDocumentTemplates() ;
      for(int i = 0; i < documentTemplates.size(); i ++) {
        String nodeTypeName = documentTemplates.get(i).toString() ;
        uicomp.options_.add(new SelectItem(nodeTypeName, nodeTypeName)) ;
      }
      uicomp.uiSelect_.setOptions(uicomp.options_) ;
    }
  }
  
  static public class SaveMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectMetadataForm uiForm = (UISelectMetadataForm) event.getSource();
      List<String> selected = new ArrayList<String>() ;
      for(int i = 0 ; i < uiForm.propertyList_.size() ; i ++) {
        if(uiForm.getUICheckBox(uiForm.propertyList_.get(i)).getSelect()) {
          selected.add(uiForm.propertyList_.get(i)) ;
        }
      }
      UIAdvanceConstraints advanceSearch = (UIAdvanceConstraints)uiForm.getSibling(UIAdvanceConstraints.class) ;
      advanceSearch.updateMetadata(selected) ;
      uiForm.setRenderedSibling(UIAdvanceConstraints.class) ;
    }
  }
  
  static public class CancelMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISelectMetadataForm uiForm = (UISelectMetadataForm) event.getSource();
      uiForm.setRenderedSibling(UIAdvanceConstraints.class) ;
    }
  }
}