/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.search.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.queries.QueryService;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id$
 */
public class UISimpleSearch extends UISimpleForm  {

  public static final String SEARCH_ACTION = "searchAction";
  public static final String MORE_CONSTRAINTS_ACTION = "moreConstraints" ;
  public static final String METADATA_PROPERTY = "metadataProperty" ;
  
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  
  private static final String ROOT_SQL_QUERY = "select * from nt:base where contains(*, '$1')";
  private static final String SQL_QUERY = "select * from nt:base where jcr:path like '$0/%' and contains(*, '$1')";
  
  private static final String ROOT_PATH_SQL_QUERY = "select * from nt:base where jcr:path like '%/$1' ";
  private static final String PATH_SQL_QUERY = "select * from nt:base where jcr:path like '$0/%/$1' ";
  
  private Node node_;
  protected UIStringInput inputText_;
  protected QueryService queryService_;
  protected String query_;
  protected String pathQuery_;
  private List<String> constraints_ = new ArrayList<String>() ;

  public UISimpleSearch(QueryService queryService) throws Exception {
    super("UISimpleSearch", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    queryService_ = queryService;    
    inputText_ = new UIStringInput("text", "");
    add(inputText_);
    
    addActionListener(SearchActionListener.class, SEARCH_ACTION);
    addActionListener(MoreConstraintsActionListener.class, MORE_CONSTRAINTS_ACTION);
    addActionListener(DeleteActionListener.class, DELETE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public String getMoreConstraintsParams() { return MORE_CONSTRAINTS_ACTION ;  }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ;  }
  
  public void updateAdvanceConstraint(String constraint) { 
    if(constraint.length() > 0) constraints_.add(" AND (" + constraint + " ) ") ;    
  }
  
  public List getConstraints() { return constraints_ ; }  
  public String getSearchAction() { return SEARCH_ACTION ; }
  public void updateNode(Node node) throws Exception {
    node_ = node;
    if ("/".equals(node_.getPath()))  query_ = ROOT_SQL_QUERY;
    else query_ = StringUtils.replace(SQL_QUERY, "$0", node_.getPath());
    
    if ("/".equals(node_.getPath()))  pathQuery_ = ROOT_PATH_SQL_QUERY;
    else if(node_.getParent().getPath().equals("/")) pathQuery_ = StringUtils.replace(PATH_SQL_QUERY, "$0", "");
    else pathQuery_ = StringUtils.replace(PATH_SQL_QUERY, "$0", node_.getParent().getPath());
  }
  
  static public class SearchActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISimpleSearch uiForm = (UISimpleSearch) event.getSource();
      String text = uiForm.inputText_.getValue();
      UIJCRExplorer explorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      Session session = explorer.getSession();
      QueryManager queryManager = session.getWorkspace().getQueryManager();      
      Information iprovider = findInformationProvider(uiForm);
      String statement = StringUtils.replace(uiForm.query_, "$1", text) ;
      String statementPath = StringUtils.replace(uiForm.pathQuery_, "$1", text) ;
      try {
        for(String constraint : uiForm.constraints_) {
          statement = statement + constraint ;
        }
        Query query = queryManager.createQuery(statement, Query.SQL);                
        Query pathQuery = queryManager.createQuery(statementPath, Query.SQL);
        long beforeTime = System.currentTimeMillis();
        QueryResult queryResult = query.execute();
        QueryResult pathQueryResult = pathQuery.execute();
        long duration = System.currentTimeMillis() - beforeTime;
        UIResults results = (UIResults) uiForm.getSibling(UIResults.class);
        results.resultMap_.clear() ;
        results.setQueryResults(duration, pathQueryResult);
        results.setQueryResults(duration, queryResult);
        uiForm.inputText_.setValue("") ;
        uiForm.constraints_.clear() ;
        uiForm.setRenderedSibling(UIResults.class);
      } catch (Exception e) {
        //e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);        
      }
    }
  }
  
  static public class MoreConstraintsActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UISimpleSearch uiForm = (UISimpleSearch) event.getComponent() ;
      UISearch uiSearch = (UISearch)uiForm.getParent() ;
      uiSearch.setRenderedSibling(UIAdvanceConstraints.class) ;
    }
  }
  
  static public class DeleteActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UISimpleSearch uiForm = (UISimpleSearch) event.getComponent() ;
      String index = event.getParameter(OBJECTID) ;
      uiForm.constraints_.remove(Integer.parseInt(index)) ;
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UISimpleSearch uiForm = (UISimpleSearch) event.getComponent() ;
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode();
      uiExplorer.broadcastOnChange(node);      
    }
  }
  
}
