/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: UIAddSignature.java 11689 2007-01-09 08:06:53Z hoapham $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.portlets.content.signatures.component;

import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDelegate;
import org.exoplatform.faces.core.component.UIHiddenInput;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.appletconfig.AppletConfigurationService;
import org.exoplatform.services.portletcontainer.ExoPortletRequest;
import org.exoplatform.services.signer.DigitalSignatureService;

/**
 * @version $Rev: 11689 $ $Date: 2007-01-09 15:06:53 +0700 (Tue, 09 Jan 2007) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class UIAddSignature extends UISimpleForm implements ChangeNodeListener {

    private static final String EXO_SIGNER_APPLET_ID = "eXoSigner";
	private static final String EMPTY = "";
	private static final String SIGN_ACTION = "sign";

    private CMSDelegate delegate;
	private Node node_;
	private Session session_;

	private DigitalSignatureService signatureSrv;
	private AppletConfigurationService appletConfigSrv;

	private UIHiddenInput contentDigest_, signature_, certificationChain_;
	private boolean supportSignature_;

    /**
     *
     */
    public UIAddSignature() {
    	super("SignatureForm", "post", null);

        setId("UIAddSignature");
        setRendererType("SimpleFormVelocityRenderer");

        PortalContainer pc = PortalContainer.getInstance();

        signatureSrv = (DigitalSignatureService) pc.getComponentInstanceOfType(
        		DigitalSignatureService.class);

        appletConfigSrv = (AppletConfigurationService) pc.getComponentInstanceOfType(
                AppletConfigurationService.class);

        // ----------------------- Form Fields -----------------------
        contentDigest_ = new UIHiddenInput("contentDigest", EMPTY);
        signature_ = new UIHiddenInput("signature", EMPTY);
        certificationChain_ = new UIHiddenInput("certificationChain", EMPTY);

        add(contentDigest_);
        add(signature_);
        add(certificationChain_);

        addActionListener(SignActionListener.class, SIGN_ACTION);
    }

    public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    	this.node_ = node;
    	this.session_ = uiExplorer.getSession();

        if (this.delegate == null)
            this.delegate = new CMSDelegate(node);
        else
            this.delegate.setNode(node);


    	setRendered(false);
    	reset();
    	loadPresetValues();
    }

    public String getAppletEmbedding() {


        ExoPortletRequest request = (ExoPortletRequest) FacesContext
                .getCurrentInstance().getExternalContext().getRequest();

        // -------------------- completes applet parameters --------------------
        Map<String,String> dynamicParams = new HashMap<String,String>();
        dynamicParams.put("algorithm", getSignatureAlgorithm());
        dynamicParams.put("signingFormName", this.getFormName());
        dynamicParams.put("signatureField", signature_.getName());
        dynamicParams.put("certificationChainField", certificationChain_.getName());
        dynamicParams.put("encodedDigestField", contentDigest_.getName());

        StringBuffer baseUrl = new StringBuffer() ;
        String portalName;
        {
            String hostName =request.getServerName();
            String port = Integer.toString(request.getServerPort());
            String protocol = request.getScheme();
            portalName = "/" + getPortalName();

            baseUrl.append(protocol).append("://").append(hostName).append(":")
                    .append(port).append(portalName);
        }
        dynamicParams.put("codeBaseUrl", baseUrl.toString());

        dynamicParams.put("archive", portalName + "/applet/launcher/exo-platform.applet-framework.core.jar," +
                portalName + "/applet/launcher/exo-platform.applet-framework.commons.jar");

        // -------------------- retrieve applet embed html ---------------------

        Map headers = FacesContext.getCurrentInstance().getExternalContext().getRequestHeaderMap();
        String ua = (String) headers.get("User-Agent");
        String html = appletConfigSrv.getAppletConfigHTMLEmbed("signer",
                getAppletId(), ua, dynamicParams);

        return html;
    }

    private String getPortalName() {
        return delegate.getPortalName();
    }

    private void loadPresetValues() {

    	// verify that current node supports signature
        try {
        	String nodeType = node_.getPrimaryNodeType().getName();
        	supportSignature_ = signatureSrv.hasNodeTypeSupport(nodeType);

        } catch (RepositoryException e) {
        	supportSignature_ = false;
        }

    	// set content digest hidden field with computed content digest
    	String encodedHash = "";

    	try {
    		if (supportSignature_) {
    			encodedHash = signatureSrv.computeDigest(node_);
    		}
        } catch (Exception e) {
	        supportSignature_ = false;
        }

    	contentDigest_.setValue(encodedHash);
    }

	public void reset() {
    	contentDigest_.setValue(EMPTY);
    	signature_.setValue(EMPTY);
    	certificationChain_.setValue(EMPTY);
    }

    public Node getNode_() {
	    return node_;
    }

    public boolean hasSignatureSupport() {
    	return supportSignature_;
    }

    public String getSignAction() {
    	return SIGN_ACTION;
    }

    public String getSignatureAlgorithm() {
    	// TODO retrieve signature algorithm from service
    	return "SHA1withRSA";
    }

    public String getAppletId() {
        return EXO_SIGNER_APPLET_ID;
    }

    private void addSignature(String encodedSignature, String encodedCertChain) {
	    try {
	        signatureSrv.addSignature(node_, encodedSignature, encodedCertChain);
        } catch (RepositoryException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
        }
    }

    public static class SignActionListener extends ExoActionListener{
        public void execute(ExoActionEvent event) throws Exception {
        	UIAddSignature uiForm = (UIAddSignature) event.getSource();
          Information iprovider = findInformationProvider(uiForm);
          ResourceBundle res = getApplicationResourceBundle() ;
          String signature = uiForm.signature_.getValue();
        	String certChain = uiForm.certificationChain_.getValue();
        	uiForm.addSignature(signature, certChain);
        	UIJCRExplorer uiExplorer = (UIJCRExplorer)uiForm.getAncestorOfType(UIJCRExplorer.class) ;
          Object[] args = { uiExplorer.getSelectNode().getName() };
          uiExplorer.setSelectNode(uiExplorer.rewind());
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
              res.getString("UIDocumentInfo.msg.signature-action-success"), args));
        }
    }
}
