/*
 * Created on Apr 8, 2005
 */
package org.exoplatform.portlets.content.templates.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.RepositoryService;


/**
 * @author benjaminmestrallet
 */
public class UINodeTypeForm extends UISimpleForm {
  
  static String NODE_TYPE_NAME = "nodetype".intern(), DIALOG = "dialog".intern();
  static String VIEW = "view".intern(), IS_DOCUMENT_TEMPLATE = "isDocumentTemplate";
  static String NODE_TYPE_LABEL = "label".intern();
  
  protected UITextArea view_;
  protected UITextArea dialog_;
  protected UISelectBox nodeTypeNameBox_ ;
  protected UIStringInput nodeTypeLabel_ ;
  protected TemplateService templateService_;
  
  public UINodeTypeForm(TemplateService templateService ) throws Exception {
    super("nodetypeForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    templateService_ = templateService;
    nodeTypeNameBox_ = new UISelectBox(NODE_TYPE_NAME, "", new ArrayList<SelectItem>()) ; 
    add(nodeTypeNameBox_);    
    nodeTypeLabel_ = new UIStringInput(NODE_TYPE_LABEL, "");
    add(nodeTypeLabel_);
    add(new UICheckBox(IS_DOCUMENT_TEMPLATE,"")) ;
    dialog_ = new UITextArea(DIALOG, "");
    dialog_.setClazz("large");
    add(dialog_);
    view_ = new UITextArea(VIEW, "");
    view_.setClazz("large");
    add(view_);
    
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public void updateNodeTypeOptions() throws Exception {
    RepositoryService repositoryService = 
      (RepositoryService)PortalContainer.getComponent(RepositoryService.class) ;   
    Session session = repositoryService.getRepository().login() ;
    NodeTypeManager nodeTypeManager = session.getWorkspace().getNodeTypeManager() ;    
    List<SelectItem> options = new ArrayList<SelectItem>();
    for(NodeTypeIterator iter = nodeTypeManager.getAllNodeTypes(); iter.hasNext();) {
      NodeType nodeType = iter.nextNodeType();
      String nodeTypeName = nodeType.getName();
      options.add(new SelectItem(nodeTypeName, nodeTypeName));
    }
    nodeTypeNameBox_.setOptions(options) ;
  }
  
  public boolean isExistedItem(String nodeType) throws Exception {
    UINodeTypesList nodeTypesList = (UINodeTypesList)getSibling(UINodeTypesList.class) ;
    for(NodeIterator iter = nodeTypesList.getNodeTypeIterator(); iter.hasNext();) {
      Node node = iter.nextNode() ;
      if(node.getName().equals(nodeType)) return true ;
    }    
    return false ;
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getComponent();
      String name = uiForm.getUISelectBox(NODE_TYPE_NAME).getValue();
      String label = uiForm.nodeTypeLabel_.getText();
      String dialog = uiForm.dialog_.getValue();
      String view = uiForm.view_.getValue(); 
      boolean isDocumentTemplate = uiForm.getUICheckBox(IS_DOCUMENT_TEMPLATE).getSelect() ;
      ResourceBundle res = getApplicationResourceBundle() ;      
      Information iprovider = findInformationProvider(uiForm);
      if(uiForm.isExistedItem(name)) {
        Object[] arg = {name} ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
                res.getString("UINodeTypeForm.msg.item-exist"), arg));
        return ;
      }
      if(label == null || label.length() < 1 ) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
            res.getString("UINodeTypeForm.msg.empty-label")));
        return ;
      }
      String[] roles = {"*"}; 
      uiForm.templateService_.addTemplate(true, name, label, isDocumentTemplate, TemplateService.DEFAULT_DIALOG, roles, dialog);
      uiForm.templateService_.addTemplate(false, name, label, isDocumentTemplate, TemplateService.DEFAULT_VIEW, roles, view);
      UINodeTypesList nodeTypesList = (UINodeTypesList)uiForm.getSibling(UINodeTypesList.class) ;
      nodeTypesList.refresh();
      uiForm.setRenderedSibling(UINodeTypesList.class);            
    }
  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getComponent();
      uiForm.setRenderedSibling(UINodeTypesList.class);
    }
  }
  
}