/*
 * Created on Apr 8, 2005
 */
package org.exoplatform.portlets.content.templates.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.NodeIterator;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * @author benjaminmestrallet
 */
public class UINodeTypesList extends UIExoCommand{
  
  static public String DELETE_NODE_TYPE = "deleteNodeType" ;
  static public String VIEW_NODE_TYPE = "viewNodeType" ;
  static public String ADD_NODE_TYPE = "addNodeType" ;
  
  static Parameter[] DELETE_NODE_TYPE_PARAMS = { new Parameter(ACTION , DELETE_NODE_TYPE)} ;
  static Parameter[] VIEW_NODE_TYPE_PARAMS = { new Parameter(ACTION , VIEW_NODE_TYPE)} ;
  static Parameter[] ADD_NODE_TYPE_PARAMS = { new Parameter(ACTION , ADD_NODE_TYPE)} ;
  static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
  
  protected TemplateService templateService_;
  public UIPageListIterator uiPageIterator_ ;  
  
  public UINodeTypesList( TemplateService templateService) throws Exception {
    setId("UINodeTypeList") ;
    setRendererType("VelocityRenderer") ;  
    templateService_ = templateService ;    
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageIterator_);
    addActionListener(DeleteNodeTypeActionListener.class, DELETE_NODE_TYPE)  ;
    addActionListener(ViewNodeTypeActionListener.class, VIEW_NODE_TYPE)  ;
    addActionListener(AddNodeTypeActionListener.class, ADD_NODE_TYPE)  ;
    addActionListener(BackActionListener.class,BACK_ACTION);
    uiPageIterator_.setPageList(new ContentPageList(getNodeTypes()));
  }
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public List getNodeTypes() throws Exception {   
    List list = new ArrayList() ;       
    Node templatesHome = templateService_.getTemplatesHome() ;
    NodeIterator nodes = templatesHome.getNodes();
    while (nodes.hasNext()) {
      Node node = nodes.nextNode();      
      list.add(node);
    }
    return list;    
  }    
  
  public NodeIterator getNodeTypeIterator() throws Exception {   
    Node templatesHome = templateService_.getTemplatesHome() ;
    return templatesHome.getNodes();
  }  
  
  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getNodeTypes()));        
  }
  
  public Parameter[] getDeleteNodeTypeParams() { return DELETE_NODE_TYPE_PARAMS ; }
  public Parameter[] getViewNodeTypeParams() { return VIEW_NODE_TYPE_PARAMS ; }
  public Parameter[] getAddNodeTypeParams() { return ADD_NODE_TYPE_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  
  static public class DeleteNodeTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypesList uicomp = (UINodeTypesList) event.getComponent() ;
      String nodeTypeName = event.getParameter(OBJECTID);      
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uicomp);
      Object[] args = { nodeTypeName };
      try {                     
        uicomp.templateService_.removeManagedNodeType(nodeTypeName) ;        
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UITemplatesList.msg.template-remove-success"), args));
        uicomp.refresh();
      }catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }                  
    }
  }
  
  static public class ViewNodeTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypesList uicomp = (UINodeTypesList) event.getComponent() ;
      String nodetypeName = event.getParameter(OBJECTID);      
      UIListsManager manager = (UIListsManager) uicomp.getSibling(UIListsManager.class);
      manager.setNodeType(nodetypeName);
      uicomp.setRenderedSibling(UIListsManager.class);
    }
  }
  
  static public class AddNodeTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypesList uicomp = (UINodeTypesList) event.getComponent() ;
      UINodeTypeForm nodeTypeForm = (UINodeTypeForm)uicomp.getSibling(UINodeTypeForm.class) ;
      nodeTypeForm.updateNodeTypeOptions() ;
      nodeTypeForm.reset() ;
      uicomp.setRenderedSibling(UINodeTypeForm.class) ;      
    }
  }
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypesList uicomp = (UINodeTypesList) event.getComponent() ;  
      UITemplatesManager parent = (UITemplatesManager)uicomp.getParent();
      parent.setRenderedSibling(UIECMAdminSummary.class);     
    }
  } 
  
}
