/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.versions.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.Version;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.cms.CMSDelegate;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.cms.CMSViewComponent;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.contentbrowser.component.CBUtils;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.RepositoryService;


/**
 * Created by The eXo Platform SARL
 * Author : Hoa Pham
 *					hoa.pham@exoplatform.com
 *          phamvuxuanhoa@yahoo.com
 * May 30, 2006
 */
public class UIViewVersion extends UIExoCommand implements ChangeNodeListener, CMSViewComponent {
  
  static String CHANGE_NODE_ACTION = "changeNode".intern();
  
  static Parameter[] CHANGE_NODE_PARAMS = { new Parameter(ACTION, CHANGE_NODE_ACTION) };
  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION, CANCEL_ACTION) };
  
  private Node parentNode_ ;
  private Node selectedNode_ ;
  private Version selectedVersion_ ;
  private Node frozenNode_ ;
  private Node versionedNode_ ;
  private CMSDelegate delegate_ ;  
  private boolean isViewByTemplate_ = false ;
  
  public UIViewVersion() {
    setId("DetailVersion") ;    
    setRendererType("VelocityRenderer") ;
    
    addActionListener(ChangeNodeActionListener.class,CHANGE_NODE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
  }
  
  public Parameter[] getChangeNodeParams() { return CHANGE_NODE_PARAMS; }  
  public Parameter[] getCancelParams() { return CANCEL_PARAMS; }
   
  public Node getParentNode() { return parentNode_ ; }
  public Node getSelectedNode() {return selectedNode_ ; }
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {   
    setRendered(false) ;
  }
  
  public void setInitData(Version selectedVersion, Node versionedNode) throws Exception {       
    selectedVersion_ = selectedVersion ;
    frozenNode_ = selectedVersion_.getNode("jcr:frozenNode") ;    
    selectedNode_ = null ;    
    versionedNode_ = versionedNode ;
    if(frozenNode_.hasNodes()) {
      parentNode_ = frozenNode_ ;
    } 
    setNode(versionedNode) ;
  }      
  
  public boolean isViewTemplateSupported() {
    boolean supported = isNodeTypeSupported() ;
    isViewByTemplate_ = supported ;
    return supported ;
  }
  public boolean hasSelectedNode() {
    if(selectedNode_ == null) return false ;
    return true ;
  }
  
  public String getViewToParse() throws Exception {
    if(isNodeTypeSupported()) {
      return "jcr:" + getDocumentTemplatePath() ; 
    }    
    return null ;
  }
  
  public boolean hasChildren() throws Exception {
    return frozenNode_.hasNodes() ;
  } 
  
  public String[] getVersionLabels() throws Exception{
    String[] labels = versionedNode_.getVersionHistory().getVersionLabels(selectedVersion_) ;
    if(labels.length>0) return labels ;
    return new String[] {"N/A"} ;
  }
  
  public String getVersionName() throws Exception {
    return selectedVersion_.getName() ;    
  }    
  
  public String getVersionedNodeName() throws Exception {       
    return versionedNode_.getName() ;
  }
  
  public String getVersionCreatedDate() throws Exception {
    return selectedVersion_.getCreated().getTime().toString();
  }
  
  public String getVersionedPrimaryNodeType() throws Exception {
    String versionedNodeType = frozenNode_.getProperty("jcr:frozenPrimaryType").getString() ;
    return versionedNodeType ;
  }
  
  public String getVersionedNodeTypeIcon() throws Exception {
    String versionedNodeType = frozenNode_.getProperty("jcr:frozenPrimaryType").getString() ;
    return CMSUtils.getIconByNodeTypeName(versionedNodeType) ;
  }
    
  public List<Property> getVersionedNodeProperties() throws Exception{
    RepositoryService repositoryService = 
      (RepositoryService)PortalContainer.getComponent(RepositoryService.class) ;
    List<Property> list = new ArrayList<Property>() ;
    NodeTypeManager nodeTypeManager = repositoryService.getRepository().getNodeTypeManager() ;
    NodeType jcrFrozenNode = nodeTypeManager.getNodeType("nt:frozenNode") ;        
    NodeType ntVersion = nodeTypeManager.getNodeType("nt:version") ;
    NodeType ntVersionHistory = nodeTypeManager.getNodeType("nt:versionHistory") ;
    NodeType mixVersionable = nodeTypeManager.getNodeType("mix:versionable") ;
    for(PropertyIterator propertyIter = frozenNode_.getProperties(); propertyIter.hasNext() ;) {
      Property property = propertyIter.nextProperty() ;      
      boolean isDefinition = false ;
      for(PropertyDefinition propDef:jcrFrozenNode.getPropertyDefinitions()) {
        if(propDef.getName().equals(property.getName())) isDefinition = true ;
      }
      for(PropertyDefinition propDef:ntVersion.getPropertyDefinitions()) {
        if(propDef.getName().equals(property.getName())) isDefinition = true ;
      }
      for(PropertyDefinition propDef:ntVersionHistory.getPropertyDefinitions()) {
        if(propDef.getName().equals(property.getName())) isDefinition = true ;
      }
      for(PropertyDefinition propDef:mixVersionable.getPropertyDefinitions()) {
        if(propDef.getName().equals(property.getName())) isDefinition = true ;
      }
      if(!isDefinition) list.add(property) ;
    }
    return list ;
  }
  
  public String getPropertyValue(Property property) throws Exception{    
    switch(property.getType()) {
    case PropertyType.BINARY: return Integer.toString(PropertyType.BINARY) ; 
    case PropertyType.BOOLEAN :return Boolean.toString(property.getValue().getBoolean()) ;
    case PropertyType.DATE : return property.getValue().getDate().getTime().toString() ;
    case PropertyType.DOUBLE : return Double.toString(property.getValue().getDouble()) ;
    case PropertyType.LONG : return Long.toString(property.getValue().getLong()) ;
    case PropertyType.NAME : return property.getValue().getString() ;
    case PropertyType.STRING : return property.getValue().getString() ;
    case PropertyType.REFERENCE : {
      if(property.getName().equals("exo:category") 
          || property.getName().equals("exo:relation")) {
        Session session = getSystemtSession() ;
        Node referenceNode = session.getNodeByUUID(property.getValue().getString()) ;
        return referenceNode.getPath() ;
      }
      return property.getValue().getString() ;
    }
    }
    return null ;
  }
  
  
  public List<String> getPropertyMultiValues(Property property) throws Exception {
    String propName = property.getName() ;    
    if(propName.equals("exo:category")) return getCategoriesValues(property) ;
    else if(propName.equals("exo:relation")) return getRelationValues(property) ;    
    List<String> values = new ArrayList<String>() ;
    for(Value value:property.getValues()) {
      values.add(value.getString()) ;
    }  
    return values ;
  }
  
  public boolean isMultiValue(Property prop) throws Exception{
    PropertyDefinition propDef = prop.getDefinition() ;
    return propDef.isMultiple() ;    
  }
  
  private List<String> getReferenceValues(Property property) throws Exception {
    Session session = getSystemtSession() ;
    List<String> pathList = new ArrayList<String>() ;
    Value[] values = property.getValues() ;
    for(Value value:values) {
      Node referenceNode = session.getNodeByUUID(value.getString()) ;
      pathList.add(referenceNode.getPath()) ;
    }
    return pathList ;
  }
  
  private List<String> getRelationValues(Property relationProp) throws Exception {
    return getReferenceValues(relationProp) ;
  }  
  
  private List<String> getCategoriesValues(Property categoryProp) throws Exception {
    return getReferenceValues(categoryProp) ;
  }
  
  public List<Node> getParentChildrenNodes() throws Exception{
    List<Node> childrenList = new ArrayList<Node>() ;
    if(parentNode_ == null) return childrenList ;
    for(NodeIterator iter = parentNode_.getNodes() ; iter.hasNext() ;) {
      childrenList.add(iter.nextNode()) ;
    }
    return childrenList ;
  }
  
  public List<Node> getSelectedChildrenNodes() throws Exception{
    List<Node> childrenList = new ArrayList<Node>() ;
    if(selectedNode_ == null) return childrenList ;
    for(NodeIterator iter = selectedNode_.getNodes() ; iter.hasNext() ;) {
      childrenList.add(iter.nextNode()) ;
    }
    return childrenList ;
  }
  
  public void changeNode(String nodePath) throws Exception {
    if (nodePath != null && nodePath.length() > 0) {
      selectedNode_ = (Node) getSystemtSession().getItem(nodePath);
      if(frozenNode_.getPath().equals(selectedNode_.getPath()))
        return;
      if (selectedNode_.getParent().getPath() != null) {
        parentNode_ = selectedNode_.getParent();       
      } else {
        parentNode_ = selectedVersion_;               
      }      
    } else {
      selectedNode_ = null;
      parentNode_ = frozenNode_;
    }     
    if(selectedNode_ != null) {
      if(delegate_ == null) {
        delegate_ = new CMSDelegate(selectedNode_) ;
      }else {
        delegate_.setNode(selectedNode_) ;
      }
    }
  }   
  
  public String getSmallNodeTypeIcon(Node node) {
    return delegate_.getSmallNodeTypeIcon(node);
  }
  
  public Node getNode() throws Exception {
    if(isViewByTemplate_) return frozenNode_ ;
    return delegate_.getNode();
  }

  public void setNode(Node node) {
    if(delegate_ == null)
      delegate_ = new CMSDelegate(node);
    else
      delegate_.setNode(node);
  }

  public String getPortalName() {
    return delegate_.getPortalName();
  }

  public String getWorkspaceName() throws Exception {    
    return delegate_.getWorkspaceName();
  }

  public String getNodeUri(Node node) throws Exception {
    return getWorkspaceName() + CBUtils.COLON + node.getPath();
  }

  public boolean isNodeTypeSupported() {
    return delegate_.isNodeTypeSupported();
  }

  public WikiContext getWikiContext() {
    return delegate_.getWikiContext();
  }

  public WikiEngineService getWikiService() {
    return delegate_.getWikiService();
  }

  public String getNodeType() throws Exception {
    return delegate_.getNodeType();
  }

  public String getDocumentTemplatePath() throws Exception {
    return delegate_.getTemplatePath();
  }
  
  public String getAttachmentSize(Node node) {
    return delegate_.getAttachmentSize(node);
  }

  public List<Node> getAttachments() throws Exception {
    return delegate_.getAttachments();
  }

  public Object getComponentInstanceOfType(String className) {
    return delegate_.getComponentInstanceOfType(className);
  }

  public List<Node> getRelations() throws Exception {
    return delegate_.getRelations();
  }

  public String getRssLink() { return null ; }

  public List getSupportedLocalise() throws Exception {
    List local = new ArrayList() ;
    return local;
  }

  public String getTemplatePath() throws Exception {
    return delegate_.getTemplatePath();
  }

  public boolean hasPropertyContent(Node node, String property){
    return delegate_.hasPropertyContent(node, property) ;
  }

  public boolean isRssLink() { return false ; }   
  
  private Session getSystemtSession() throws Exception {
    RepositoryService repositoryService = 
      (RepositoryService)PortalContainer.getComponent(RepositoryService.class) ;
    CmsConfigurationService cmsConfigService = 
      (CmsConfigurationService) PortalContainer.getComponent(CmsConfigurationService.class) ;
    Session session = repositoryService.getRepository().getSystemSession(cmsConfigService.getWorkspace()) ;
    return session ;
  }
  
  
  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIViewVersion uiComp = (UIViewVersion) event.getSource();
      String nodePath = event.getParameter(OBJECTID);
      uiComp.changeNode(nodePath);
    }
  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {   
      UIViewVersion uiComp = (UIViewVersion) event.getSource();
      uiComp.setRenderedSibling(UIVersionInfo.class) ;
    }
  }

  public String getLargeNodeTypeIcon(Node node) {
    return delegate_.getLargeNodeTypeIcon(node);
  }

  public String getPropertyIconByName(String propertyName) {
    return delegate_.getPropertyIconByName(propertyName);
  }

  public String getPropertyTypeIcon(Property propety) {
    return delegate_.getPropertyTypeIcon(propety);
  }  
}
