package org.exoplatform.portlets.content.views.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.views.ManageViewService;


public class UITabForm extends UISimpleForm {
  
  private static String ADD_TAB_ACTION = "addTab" ;
  public static Parameter[] ADD_TAB_PARAMS = { new Parameter(ACTION , ADD_TAB_ACTION)} ;
  List buttons_ = new ArrayList() ;
  boolean isView_ = false ;  
  private Node view_ = null ;
  ManageViewService vservice_ ;
  public UITabForm(ManageViewService vservice) throws Exception {    
    super("uiTabForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    vservice_ = vservice ;
    UIStringInput tab = new UIStringInput("name","") ;
    add(tab) ;
    buttons_ = vservice.getButtons() ;
    for(int i = 0 ; i < buttons_.size() ; i ++ ){
      UICheckBox checkbox = new UICheckBox(buttons_.get(i).toString(), "") ;
      add(checkbox) ;
    }
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }
  
  public boolean isView() { return isView_ ; }
  public List getButtons() { return buttons_ ; }
  public void refresh(Node view){
    vservice_ = (ManageViewService) PortalContainer.getInstance()
                .getComponentInstanceOfType(ManageViewService.class) ;
    getUIStringInput("name").setValue("") ;
    getUIStringInput("name").setEditable(true) ;
    for(int i = 0 ; i < buttons_.size() ; i ++ ){
      getUICheckBox(buttons_.get(i).toString()).setSelect(false) ;
      getUICheckBox(buttons_.get(i).toString()).setEditable(true) ;
    }
    isView_ = false ;    
    view_ = view ;    
  }
  
  public void update(Node tabNode, boolean isView) throws Exception{    
    getUIStringInput("name").setValue(tabNode.getName()) ;
    String buttonsProperty = tabNode.getProperty("exo:buttons").getString() ;
    String[] buttonArray = StringUtils.split(buttonsProperty, ";") ;
    
    for(int i = 0 ; i < buttonArray.length ; i ++ ){
      getUICheckBox(buttonArray[i].trim()).setSelect(true) ;        
    }
    if(isView){
      getUIStringInput("name").setEditable(false) ;
      for(int j = 0 ; j < buttons_.size() ; j ++ ){
        getUICheckBox(buttons_.get(j).toString()).setEditable(false) ;
      }
    }else{
      getUIStringInput("name").setEditable(true) ;
      for(int j = 0 ; j < buttons_.size() ; j ++ ){
        getUICheckBox(buttons_.get(j).toString()).setEditable(true) ;
      }
    }
    isView_ = isView ;
  }
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITabForm uiForm = (UITabForm)event.getSource() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiForm) ;
      
      String tabName = uiForm.getUIStringInput("name").getValue() ;
      if(tabName == null || tabName.length() < 1){
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
            res.getString("UITabForm.msg.tab-name-error"))) ;
        return ;
      }
       
      StringBuffer selectedButton = new StringBuffer("") ;
      boolean isSelected = false ;
      for(int i = 0 ; i < uiForm.buttons_.size() ; i ++ ) {
        String button = uiForm.buttons_.get(i).toString() ;
        if(uiForm.getUICheckBox(button).getSelect()){
          isSelected = true ;
          if(selectedButton.length() > 0)
            selectedButton.append(";").append(button) ;
          else
            selectedButton.append(button) ;
        }
      }
      if(!isSelected){
        iprovider.addMessage(new ExoFacesMessage(res.getString("UITabForm.msg.button-select-error"))) ;
        return ;
      }
      
      UIViewFormController viewFormController = (UIViewFormController)uiForm.getSibling(UIViewFormController.class) ;
      UIViewForm viewForm = (UIViewForm) viewFormController.getChildComponentOfType(UIViewForm.class);      
      if(uiForm.view_ != null){        
        uiForm.vservice_.addTab(uiForm.view_ , tabName, selectedButton.toString()) ;            
        viewFormController.showEditViewForm(uiForm.view_, false);
      }else{        
        viewForm.addNewTab(tabName, selectedButton.toString()) ;                     
      }                        
      uiForm.setRenderedSibling(UIViewFormController.class) ;
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITabForm uiForm = (UITabForm)event.getSource() ;
      uiForm.setRenderedSibling(UIViewFormController.class) ;
    }
  }
}
