/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: BaseDigitalSigner.java 9514 2006-10-16 21:36:34Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.signer.impl;

import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.util.List;

import org.apache.commons.logging.Log;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.signer.DigitalSigner;

/**
 * DigitalSigner plugin base class used to develop new supported nodetypes
 * signing.
 * 
 * @version $Rev: 9514 $ $Date: 2006-10-17 04:36:34 +0700 (Tue, 17 Oct 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public abstract class BaseDigitalSigner extends BaseComponentPlugin implements
        DigitalSigner {

	/**
	 * Digital Signer logger.
	 */
	protected Log logger;

	/**
	 * NodeType names handled by subclasses.
	 */
	private String[] nodeTypeNames;

	/**
	 * Init parameters:
	 * <ul>
	 * <li><code>signer.supported.nodetypes</code> - value list of supported
	 * nodetype names handled by the digital signer.</li>
	 * </ul>
	 * 
	 * @param params
	 *            configuration parameters.
	 * @param logSrv
	 *            system logging service.
	 */
	public BaseDigitalSigner(InitParams params, LogService logSrv) {
		super();

		logger = logSrv.getLog(DigitalSigner.class);

		List<String> initNames = params.getValuesParam(
		        "signer.supported.nodetypes").getValues();
		nodeTypeNames = initNames.toArray(new String[initNames.size()]);
	}

	/**
	 * Obtain hash value of the given data using the specified algorithm,
	 * accessing the HashFunctionService instance received on constructor.
	 * 
	 * @param data
	 *            content to generate the signature
	 * @param algorithm
	 *            a supported hash algorithm
	 * @return hash value "fingerprint"
	 * @throws IOException
	 * @throws NoSuchAlgorithmException
	 * @throws NoSuchProviderException 
	 */
	protected byte[] calculateDigest(InputStream data, String algorithm)
	        throws NoSuchAlgorithmException, NoSuchProviderException, IOException {

		JCEUtil jceUtil = JCEUtil.getInstance();
		return jceUtil.calculateDigest(data, algorithm);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.exoplatform.services.signer.DigitalSigner#getNodeTypeNames()
	 */
	public String[] getNodeTypeNames() {
		return nodeTypeNames;
	}
}
