/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: JCEUtil.java 9514 2006-10-16 21:36:34Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.signer.impl;

import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.Provider;
import java.security.Security;

/**
 * @version $Rev: 9514 $ $Date: 2006-10-17 04:36:34 +0700 (Tue, 17 Oct 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class JCEUtil {

	private static JCEUtil instance;

	/**
	 * 4096 bytes.
	 */
	private static final int READ_BLOCK_SIZE = 4096;

	private static String jceProvider;

    private JCEUtil() {
    	super();

    	try {
    		Class clazz = Class.forName("org.bouncycastle.jce.provider.BouncyCastleProvider");
    		Provider prov = (Provider) clazz.newInstance();
        	Security.addProvider(prov);
    		jceProvider = prov.getName();
        } catch (Exception e) {
    		jceProvider = null;
        }
    }

    public static JCEUtil getInstance() {
    	if (instance == null)
    		instance = new JCEUtil();
    	return instance;
    }

	public byte[] calculateDigest(InputStream data, String algorithm) throws NoSuchAlgorithmException, NoSuchProviderException, IOException {

		MessageDigest md = (jceProvider == null ? MessageDigest.getInstance(algorithm) : MessageDigest.getInstance(algorithm, jceProvider));

		int read;
		byte[] buffer = new byte[READ_BLOCK_SIZE];

		while ((read = data.read(buffer)) != -1) {
			md.update(buffer, 0, read);
		}

		buffer = null;

		return md.digest();
	}

}
