/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

import java.util.Map;

import javax.jcr.Property;
import javax.jcr.Node ;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Session;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.services.cms.scripts.CmsScript;
import org.exoplatform.services.document.DocumentReaderService;
import org.exoplatform.services.document.diff.DiffService;
import org.exoplatform.services.document.diff.Revision;
import org.exoplatform.services.jcr.RepositoryService;

public class TransformBinaryChildrenToTextScript implements CmsScript {

  private DocumentReaderService readerService_;
  private RepositoryService repoService_ ;
  private DiffService diffService_ ;

  public TransformBinaryChildrenToTextScript(DocumentReaderService readerService,RepositoryService repoService, DiffService diffService) {  
    readerService_ = readerService;
    repoService_ = repoService ;
    diffService_ = diffService ;
  }                

  public void execute(Object context) {
    Map variables = (Map) context;       

    String folderPath = (String)variables.get("srcPath") ;
    String worksapce = (String)variables.get("srcWorkspace") ;
    String actionName = (String)variables.get("actionName") ;
    String eventPath = (String)variables.get("nodePath") ;
    if(!checkLocalEvents(folderPath,eventPath)) return  ;
    try {
      Session session = repoService_.getRepository().getSystemSession(worksapce) ;           
      Node folderNode = (Node)session.getItem(folderPath) ;
      Node convertedFolder = null ;
      if(folderNode.hasNode("convertedFolder")) {
        convertedFolder = folderNode.getNode("convertedFolder") ;
      }else {
        convertedFolder = folderNode.addNode("convertedFolder","nt:folder") ;
        folderNode.save() ;
        session.save() ;
      }
      NodeIterator iter = folderNode.getNodes() ;
      while(iter.hasNext()) {
        Node childNode = iter.nextNode() ;
        if(!childNode.getPrimaryNodeType().isNodeType("nt:file")) continue ;
        String currentContent = getContentAsText(childNode) ;
        if(currentContent == null) continue ;
        String fileName = childNode.getName().concat(".txt") ;
        Node contentNode = null, file = null;
        if(convertedFolder.hasNode(fileName)) {          
            file = convertedFolder.getNode(fileName) ;            
            if(!file.isCheckedOut()) file.checkout() ;
            file.checkin();
            file.checkout() ;          
        }else {
          file = convertedFolder.addNode(fileName,"nt:file") ;
          file.addNode("jcr:content", "nt:resource");         
          file.addMixin("mix:versionable") ;                              
        }
        contentNode = file.getNode("jcr:content") ;
        contentNode.setProperty("jcr:encoding", "UTF-8");
        contentNode.setProperty("jcr:mimeType", "text/html");    
        contentNode.setProperty("jcr:data", currentContent);
        contentNode.setProperty("jcr:lastModified", new GregorianCalendar());
        convertedFolder.save() ;
      }      
      session.save() ;
      session.refresh(true) ;
    } catch (Exception e) {      
      e.printStackTrace();
    }  
}

  private boolean checkLocalEvents(String srcPath, String eventPath) {
    if(srcPath.equals(eventPath.substring(0,eventPath.lastIndexOf("/"))))
      return true ;
    return false ;
  } 
  
  private String getContentAsText(Node node) {
    Node content = node.getNode("jcr:content");
    Property mime = content.getProperty("jcr:mimeType");
    if (!mime.getString().startsWith("text")) return null ;    
    return readerService_.getContentAsText(mime.getString(), content.getProperty("jcr:data").getStream());    
  }

  public void setParams(String[] params) {}

}